const CompressionPlugin = require('compression-webpack-plugin');



const path = require("path");
 
const resolve = dir => {
  return path.join(__dirname, dir);
};
 
// 项目部署基础
// 默认情况下，我们假设你的应用将被部署在域的根目录下,
// 例如：https://www.my-app.com/
// 默认：'/'
// 如果您的应用程序部署在子路径中，则需要在这指定子路径
// 例如：https://www.foobar.com/my-app/
// 需要将它改为'/my-app/'
// iview-admin线上演示打包路径： https://file.iviewui.com/admin-dist/

const BASE_URL =
  process.env.NODE_ENV === "production" ? process.env.URL : "/";


module.exports = {
    devServer: {
        host: '127.0.0.1',
        port: 9999,
        proxy: {
            '/adminapi': {
                target: 'http://127.0.0.1:8211',  
                ws: true
            },
        }
    },

    publicPath: BASE_URL,
    // tweak internal webpack configuration.
    // see https://github.com/vuejs/vue-cli/blob/dev/docs/webpack.md
    // 如果你不需要使用eslint，把lintOnSave设为false即可
    lintOnSave: true,
    //兼容低版本浏览器不能解析es6语法，先安装需要的包 
    // npm install -D babel-loader@7 babel-core babel-preset-env webpack
    // 打包时不生成.map文件 避免看到源码
    productionSourceMap: false,
    // 部署优化
    configureWebpack: {
        // 使用CDN
        // externals: {
        //     vue: 'Vue',
        //     'vue-i18n': 'VueI18n',
        //     axios: 'axios',
        //     'vue-router': 'VueRouter',
        //     vuex: 'Vuex',
        //     'view-design': 'iview',
        //     echarts: 'echarts',
        //     apexcharts: 'ApexCharts',
        //     'vue-apexcharts': 'VueApexCharts',
        //     dplayer: 'DPlayer',
        //     'print-js': 'printJS',
        //     html2canvas: 'html2canvas',
        //     'vue-json-pretty': 'VueJsonPretty',
        //     'vue-lazyload': 'VueLazyload',
        //     'js-cookie': 'Cookies',
        //     vuedraggable: 'vuedraggable',
        //     viewerjs: 'Viewer'
        // },
        module: {
            rules: [
              {
                test: /\.m?js$/,
                //exclude用下面配置的话，默认是过滤不编译node_modules 路径下的文件
                //exclude: /(node_modules|bower_components)/,
                //include 指定需要编译的文件路径
                include: [
                  resolve("src"),
                  resolve("node_modules/tree-table-vue/lib"),
                  resolve("node_modules/v-org-tree/dist"),
                  resolve("node_modules/view-design/src/locale")// 我的是把iview 升级为4的版本,所以是view-design包，之前是iview 包
                ],
                use: {
                  loader: "babel-loader",
                  options: {
                    presets: ["@babel/preset-env"]
                  }
                }
              }
            ]
          },
        // GZIP压缩
        plugins: [
            new CompressionPlugin({
                test: /\.js$|\.html$|\.css/, // 匹配文件
                threshold: 10240 // 对超过10k文件压缩
            })
        ]
    },
    chainWebpack: config => {
        config.module.rule('md')
            .test(/\.md/)
            .use('vue-loader')
            .loader('vue-loader')
            .end()
            .use('vue-markdown-loader')
            .loader('vue-markdown-loader/lib/markdown-compiler')
            .options({
                raw: true
            })
    }
}