<?php
namespace App\Console\Commands;

use App\Finance\Repositories\Contracts\InvoiceRecordRepository;
use App\Finance\Repositories\Eloquent\InvoiceRecordRepositoryEloquent;
use App\Finance\Services\InvoiceService;
use Illuminate\Console\Command;
use Illuminate\Console\ConfirmableTrait;

class InvoiceJobCommand extends Command
{
    use ConfirmableTrait;
    /**
     * 命令行的名称及用法。
     *
     * @var string
     */
    protected $signature = 'invoice:job 
                            {--action_type= : 任务操作类型}
                            {--params= : 任务参数}';

    /**
     * 命令行的概述。
     *
     * @var string
     */
    protected $description = '财务开票任务命令行';

    /**
     * 创建新的命令实例。
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * 运行命令。
     * @throws \Exception
     */
    public function handle()
    {

        if (! $this->confirmToProceed()) {
            return 1;
        }

        if($this->hasArgument('help')){
            $this->help();
        }else{
            $action_type = $this->option('action_type');
            $params = $this->option('params');

            if(empty($action_type)){
                $this->error('缺少命令参数,请输入具体的参数命令.如需帮助请输入 --help');
                exit;
            }

            //$service = app(InvoiceService::class);
            $service = (new InvoiceService(app(InvoiceRecordRepositoryEloquent::class)));
            switch($action_type){
                case 'registerPlatform' :
                    $service->registerPlatform();
                    break;
                case 'getQiyeInvoiceInfo' :
                    $service->getQiyeInvoiceInfo();
                    break;
                case 'queryInvoiceInfo' :
                    $response = $service->queryInvoiceInfo([
                        'invoice_platform' => 'rhawn',
                    ]);
                    var_dump($response);
                    break;
                case 'testEmail' :
                    $service->testEmail('IRB220230306XYNKM');
                    break;
                case 'resetApplyInvoice' :
                    $service->resetApplyInvoice($params);
                    break;
                case 'manualDownLoadInvoicePdf' :
                    $service->manualDownLoadInvoicePdf($params);
                    break;
                case 'apiCallBack' :
                    $service->apiCallBack([
                        'invoice_platform' => 'rhawn',
                    ]);
                    break;
                case 'tryAgainInvoiceApiResult' :
                    $service->tryAgainInvoiceApiResult();
                    break;
                case 'applyInvoice' :
                    $service->applyInvoice([
                        'invoice_platform' => 'rhawn',
                        'invoice_type' => 'blue',
                        'invoice_category' => '2',
                        'invoice_platform_id' => '16712',
                        'invoice_creater_by' => '高红英',
                        'invoice_items' => [
                            [
                                'item_id' => '425152',
                                'item_name' => '四丁基溴化铵',
                                'item_price' => '62.00',
                                'item_nums' => '1',
                                'item_spec' => '罗恩 R012573-500g'
                            ],
                            [
                                'item_id' => '426141',
                                'item_name' => '丙烯酰胺',
                                'item_price' => '46.00',
                                'item_nums' => '1',
                                'item_spec' => '罗恩 R003167-500g'
                            ],
                        ],
                        'invoice_money' => '108',
                        'invoice_customer' => [
                            'cust_name' => '上海百舜生物科技有限公司',
                            'cust_code' => '91310115312498004F',
                            'cust_address' => '上海市奉贤区联合北路215号第3幢1242室',
                            'cust_phone' => '021-37581181',
                            'cust_bankname' => '交行上海大场支行',
                            'cust_bankaccount' => '310065060018010017377',
                            'cust_mobile' => '13162997331',
                            "cust_email" => "aloneshen2003@163.com"
                        ]

                    ]);
                    break;
                default:
            }
        }
    }

    public function help()
    {
        $helpStr = "参数帮助说明";
        $this->comment($this->setHelp($helpStr)->getProcessedHelp());
        $this->line("action_type: 具体动作参数\n\n initProductToExcel => 初始化批量商品 \n batchUpdateProduct => 批量更新商品\n\nparams : 操作需要传入的参数.非必填项");
    }
}
