<?php
namespace App\Console\Commands;

use App\Services\Api\RhawnOrdersService;
use App\Services\AuthService;
use App\Services\CustomerService;
use Illuminate\Console\Command;
use Illuminate\Console\ConfirmableTrait;

class TestJobCommand extends Command
{
    use ConfirmableTrait;
    /**
     * 命令行的名称及用法。
     *
     * @var string
     */
    protected $signature = 'test:job 
                            {--action_type= : 任务操作类型}
                            {--params= : 任务参数}';

    /**
     * 命令行的概述。
     *
     * @var string
     */
    protected $description = 'test任务命令行';

    /**
     * 创建新的命令实例。
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * 运行命令。
     * @throws \Exception
     */
    public function handle()
    {

        if (! $this->confirmToProceed()) {
            return 1;
        }

        if($this->hasArgument('help')){
            $this->help();
        }else{
            $action_type = $this->option('action_type');
            $params = $this->option('params');

            if(empty($action_type)){
                $this->error('缺少命令参数,请输入具体的参数命令.如需帮助请输入 --help');
                exit;
            }

            $service = app(RhawnOrdersService::class);
            switch($action_type){
                case 'test' :
                    $service->getCustomerRhawnOrdersList(['customer_code' => '80330','page_size' => 100, 'page_no' => 1]);
                    break;
                case 'getCustomerOrderDetail' :
                    $service->getCustomerOrderDetail(['customer_code' => '80330','order_number' => '20170911-1305']);
                    break;
                case 'createNewCustomerOrders' :
                    $service->createNewCustomerOrders([
                        'customer_code' => '80056',
                        "so_cus_po" => '2000000011111',
                        "so_ca_name" => '张三',
                        "so_ca_mobile" => '13321686555',
                        "so_ca_phone" => '61111111',
                        "so_ca_province" => '上海',
                        "so_ca_city" => '上海市',
                        //"so_ca_county" => '奉贤区',
                        "so_ca_street" => '望园路88888号888室',
                        "so_note" => '测试',
                        'items' => [
                            [
                                'p_code' => 'R000001-20mg',
                                'num' => 1,
                                'si_molbase_pog' => ''
                            ]
                        ]
                    ]);
                    break;
                case 'cancelCustomerOrder' :
                    $service->cancelCustomerOrder(['customer_code' => '80056','order_number' => '20220929-1225649']);
                    break;
                case 'createPlatformCustomer' :
                    $customerService = app(CustomerService::class);
                    $customerService->createPlatformCustomer(['customer_code' => '80056','customer_company' => '测试']);
                    break;
                case 'refreshCustomerToken' :
                    $customerService = app(CustomerService::class);
                    $customerService->refreshCustomerToken(['platform_customer_id' => '2']);
                    break;
                case 'getToken' :
                    $authService = app(AuthService::class);
                    $authService->getToken(['token' => '2tylDpWuvLAfWmAX1']);
                    break;
                default:
            }
        }
    }

    public function help()
    {
        $helpStr = "参数帮助说明";
        $this->comment($this->setHelp($helpStr)->getProcessedHelp());
        $this->line("action_type: 具体动作参数\n\n initProductToExcel => 初始化批量商品 \n batchUpdateProduct => 批量更新商品\n\nparams : 操作需要传入的参数.非必填项");
    }
}
