<?php
namespace App\Console\Commands;

use App\Services\ThirdPlatform\ZhenKhService;
use Illuminate\Console\Command;
use Illuminate\Console\ConfirmableTrait;

class ZhenkhJobCommand extends Command
{
    use ConfirmableTrait;
    /**
     * 命令行的名称及用法。
     *
     * @var string
     */
    protected $signature = 'zkh:job 
                            {--action_type= : 任务操作类型}
                            {--params= : 任务参数}';

    /**
     * 命令行的概述。
     *
     * @var string
     */
    protected $description = '震坤行任务命令行';

    /**
     * 创建新的命令实例。
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * 运行命令。
     * @throws \Exception
     */
    public function handle()
    {

        if (! $this->confirmToProceed()) {
            return 1;
        }

        if($this->hasArgument('help')){
            $this->help();
        }else{
            $action_type = $this->option('action_type');
            $params = $this->option('params');

            if(empty($action_type)){
                $this->error('缺少命令参数,请输入具体的参数命令.如需帮助请输入 --help');
                exit;
            }

            $service = app(ZhenKhService::class);
            switch($action_type){
                case 'test' :
                    $service->test();
                    break;
                case 'getUpdateGoods' :
                    $service->getUpdateSupplierGoods();
                    break;
                case 'getUpdateOrders' :
                    $service->getUpdateSupplierOrders();
                    break;
                case 'sendDeliveryOrder' :
                    $service->sendDeliveryOrder($params);
                    break;
                case 'getDeliveryOrderDetail' :
                    $service->getDeliveryOrderPdf($params);
                    break;
                case 'orderNoRollBack' :
                    $service->orderNoRollBack();
                    break;
                case 'createJwtToken' :
                    $service->createJwtToken();
                    break;
                case 'hMUpdateOrdersList' :
                    $service->hMUpdateOrdersList();
                    break;
                case 'batchUpdateDeliveryOrder' :
                    $service->batchUpdateDeliveryOrder($params);
                    break;
                case 'sendMail' :
                    $service->sendMail('订单确认邮件', '订单号:未确认成功');
                case 'orderConfirm' :
                    $service->orderConfirm($params);
                default:
            }

            /*$productUpdateJob = (new ZhenkhJob($action_type,$params))->delay(100);
            app('Illuminate\Contracts\Bus\Dispatcher')->dispatch($productUpdateJob);*/
        }
    }

    public function help()
    {
        $helpStr = "参数帮助说明";
        $this->comment($this->setHelp($helpStr)->getProcessedHelp());
        $this->line("action_type: 具体动作参数\n\n initProductToExcel => 初始化批量商品 \n batchUpdateProduct => 批量更新商品\n\nparams : 操作需要传入的参数.非必填项");
    }
}
