<?php

namespace App\Finance\Repositories\Eloquent;

use App\Finance\Repositories\Contracts\InvoiceRecordRepository;
use App\Finance\Repositories\Models\InvoiceRecord;

class InvoiceRecordRepositoryEloquent extends BaseRepository implements InvoiceRecordRepository
{
    protected $fieldSearchable = [
    ];

    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return InvoiceRecord::class;
    }

    public function saveInvoice($saveParams)
    {
        $result = $this->updateOrCreate(['invoice_number' => $saveParams['invoice_number']],$saveParams);
        return $result->toArray();
    }

    public function getInvoiceRecordByNumber($invoiceNumber)
    {
        $result = $this->getList(['invoice_number' => $invoiceNumber]);
        return $result;
    }

    public function getList($where = [], $offset = 0, $limit = 100)
    {
        $query = $this;
        if(!empty($where)){
            foreach($where as $whereKey => $whereValue){
                if(is_array($whereValue)){
                    $query = $query->whereIn($whereKey,$whereValue);
                }else{
                    if(strtotime($whereValue) && strtotime($whereValue) > 0){
                        $query = $query->whereDate($whereKey,'=',$whereValue);
                    }else{
                        $query = $query->where($whereKey,$whereValue);
                    }
                }
            }
        }
        $result = $query->offset($offset)->limit($limit)->orderBy('id', 'desc')->get()->toArray();
        return $result;
    }

    public function getInvoiceRecordByRealNumber($realNumber)
    {
        $result = $this->getList(['invoice_real_number' => $realNumber]);
        return $result;
    }

    public function delInvoiceByNumber($invoiceNumber)
    {
        return $this->deleteWhere([
            'invoice_number' => $invoiceNumber
        ]);
    }

    public function getNotResultInvoiceRecord()
    {
        return $this->getList(['invoice_status' => ['-1','2']]);
    }

    public function counts($where)
    {
        if(!empty($where)){
            foreach($where as $whereKey => $whereValue){
                if(strtotime($whereValue) && strtotime($whereValue) > 0){
                    unset($where[$whereKey]);
                    $where[$whereKey] = [$whereKey,'DATE',$whereValue];
                }
            }
        }
        return $this->count($where);
    }

}
