<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\Middleware;

use App\Exceptions\ServerRunTimeException;
use App\Jobs\RequestLogJob;
use App\Services\CustomerService;
use App\Services\ThirdPlatform\ThirdPlatformService;
use App\Services\ThirdPlatform\ZhenKhService;
use Closure;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Contracts\Auth\Factory as Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Support\Traits\Helpers;
use Symfony\Component\HttpFoundation\Response as SymfonyResponse;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;


class Requestlogs
{
    use Helpers;
    /**
     * The authentication guard factory instance.
     *
     * @var Auth
     */
    protected $auth;

    /**
     * Create a new middleware instance.
     *
     * @param  Auth  $auth
     */
    public function __construct(Auth $auth)
    {
        $this->auth = $auth;
    }

    /**
     * Handle an incoming request.
     *
     * @param  Request  $request
     * @param  Closure  $next
     * @param  string|null  $guard
     * @return mixed
     *
     * @throws AuthorizationException
     */
    public function handle($request, Closure $next, $guard = null)
    {
        if(!$request->header('authorization')){
            return Response::fail('token 不存在!',500,'');
        }
        $token = trim(str_ireplace('bearer', '', $request->header('authorization')));
        try{
            $decodeToken = $this->decodeToken($token);
            if(empty($decodeToken) || (!is_array($decodeToken) && !$decodeToken['hash'])){
                return Response::fail('客户token无效',500,'');
            }
        }catch(\Throwable $exception){
            return Response::fail($exception->getMessage(),500,'');
        }

        $start = $request->server->get('REQUEST_TIME_FLOAT');

        $uniqueId = $request->headers->get('X-Unique-Id') ?: Str::uuid()->toString();

        $context = [
            'request' => $request->request->all(),
            'uniqueId' => $uniqueId,
            'token' => $decodeToken['hash'],
            'start' => $start,
        ];

        $job = new RequestLogJob(\config('logging.request.message'), $context, request()->server());
        $job->handle();

        return $next($request);
    }
}
