<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\V1\Auth;

use App\Services\AuthService;
use Illuminate\Http\Request;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;
use Jiannei\Response\Laravel\Support\Facades\Response;


use OpenApi\Annotations\Post;
use OpenApi\Annotations\RequestBody;
use OpenApi\Annotations\MediaType;
use OpenApi\Annotations\Schema;
use OpenApi\Annotations\Property;
use OpenApi\Annotations\Response as AnnotationResponse;
use OpenApi\Annotations as OA;


class CustomerAuthController extends Controller
{
    use Helpers;

    public function __construct(AuthService $authService)
    {
       $this->authService = $authService;
    }

    /**
     * @Post(
     *     path="/openapi/auth/getToken",
     *     tags={"AuthToken"},
     *     summary="获取用户auth token，用于接口的用户权限验证",
     *     description="获取用户auth token",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"customerToken"},
     *                 @Property(property="customerToken", @Schema(type="string"),description="用户私有key"),
     *                 example={"customerToken":"cKCXPza6y23l_c0z"}
     *             ),
     *         )
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                 }
     *             )
     *         )
     *     ),
     * )
     */
    public function getToken(Request $request)
    {
        $message = [
            'customerToken.required' => "客户token必传",
        ];
        $this->validateRequest($request, $message);

        $params = $request->all();
        $params = $this->formatKeysfromArray($params,'toUnderScore');

        try{
            $token = $this->authService->getToken($params);
            return Response::success($token,'操作成功');
        }catch(\Throwable $exception){
            return Response::fail('获取token失败',500);
        }
    }
}
