<?php

namespace App\Http\Controllers\V1\Finance\Front;

use App\Finance\Services\InvoiceService;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

class InvoiceController extends Controller
{
    use Helpers;

    public function __construct(InvoiceService $invoiceService)
    {
       $this->invoiceService = $invoiceService;
    }

    public function getInvoiceRecordList(Request $request)
    {
        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        try{
            $result = $this->invoiceService->getInvoiceRecordList($requestParams);
            return Response::success($this->formatKeysfromArray($result),'获取成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }

    public function getInvoiceStatusDetail(Request $request)
    {
        $message = [
            'invoiceNumber.required' => "发票流水号不能为空",
        ];
        $this->validateRequest($request, $message);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        try{
            $result = $this->invoiceService->getInvoiceStatusDetailByInvoiceNumber($requestParams);
            return Response::success($this->formatKeysfromArray($result),'状态详情获取成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }

    public function invoiceRequestReset(Request $request)
    {
        $message = [
            'invoiceNumber.required' => "发票流水号不能为空",
        ];
        $this->validateRequest($request, $message);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        try{
            $result = $this->invoiceService->resetApplyInvoice($requestParams['invoice_number']);
            return Response::success($result,'重置请求成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }

    public function delInvoiceRecord(Request $request)
    {
        $message = [
            'invoiceNumber.required' => "发票流水号不能为空",
        ];
        $this->validateRequest($request, $message);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        try{
            $result = $this->invoiceService->delInvoiceRecord($requestParams);
            return Response::success($result,'请求成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }
}
