<?php

namespace App\Http\Controllers\V1\Finance;

use App\Finance\Services\InvoiceService;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

class InvoiceController extends Controller
{
    use Helpers;

    public function __construct(InvoiceService $invoiceService)
    {
       $this->invoiceService = $invoiceService;
    }

    public function applyInvoice(Request $request)
    {
        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');

        try{
            $result = $this->invoiceService->applyInvoice($requestParams);
            return Response::success($result,'发票申请成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }

    public function cancelInvoice(Request $request)
    {
        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');

        try{
            $result = $this->invoiceService->cancelBlueInvoice($requestParams);
            return Response::success($result,'发票作废申请成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }

    public function baiwangInvoiceCallBack(Request $request)
    {
        $requestContent = $request->getContent();
        if(!empty($requestContent)){
            $decodeContent = base64_decode($requestContent);
            if($decodeContent){
                $requestParams = $this->formatKeysfromArray(json_decode($decodeContent,true),'toUnderScore');
                $result = $this->invoiceService->apiCallBack($requestParams);
                if($result){
                    return response()->json([
                        'code' => '0',
                        'message' => '成功',
                        'renson' => ''
                    ]);
                }
            }
        }
    }

    public function baiwangInvoiceDownload(Request $request)
    {
        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        if(!isset($requestParams['token']) || empty($requestParams['token'])){
            return Response::fail('参数错误！',500);
        }
        $decode = base64_decode($requestParams['token']);
        list($token,$hm) = explode('&',$decode);
        list($hmKey,$hmValue) = explode('=',$hm);
        if($hmKey != 'hm' || empty($hmValue)){
            return Response::fail('参数错误！',500);
        }

        list($tokenKey,$tokenValue) = explode('=',$token);
        if($tokenKey != 'hash' || empty($tokenValue)){
            return Response::fail('参数错误！',500);
        }
        try{
            $result = $this->invoiceService->downLoadInvoice(['token' => $tokenValue,'hm' => $hmValue]);
            if($result){
                $headers = [
                    'Content-Type: application/pdf',
                ];
                //下载电子发票
                header('Content-Type: application/octet-stream');
                header("Content-Transfer-Encoding: utf-8");
                header("Content-disposition: attachment; filename=\"" . basename($result['filename']) . "\"");
                readfile($result['path']);
                //return response()->download($result['path'],$result['filename'],$headers);
            }
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }

    public function downloadInvoice(Request $request)
    {
        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');

        try{
            $result = $this->invoiceService->getInvoicePdfDownloadUrl($requestParams);
            return Response::success($result['downloadUrl'],'');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,$exception->getMessage());
        }
    }

}
