<?php

namespace App\Jobs;

use App\Services\SystemJournalService;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Monolog\Logger;
use Monolog\Processor\WebProcessor;
use Psr\Log\LoggerInterface;

class RequestLogJob extends Job
{
    use InteractsWithQueue;
    use Queueable;

    private $context;
    private $message;
    private $serverData;

    public function __construct (string $message, array $context = null, array $serverData = null)
    {
        $this->message = $message;
        $this->context = $context;
        $this->serverData = $serverData;
    }

    public function handle()
    {
        $logger = null;
        if (config('logging.request.enabled')) {
            app()->forgetInstance(LoggerInterface::class);

            $logger = app(LoggerInterface::class)->getLogger();
            if ($logger instanceof Logger) {
                $logger->pushProcessor(new WebProcessor($this->serverData));
            }
        }
        try{
            app(SystemJournalService::class)->writeToDbLog($this->context,$this->serverData);
            if ($logger) {
                $logger->debug($this->message, $this->context);
            }
        }catch(\Throwable $exception){
            $this->context['exception'] = $exception;
            if ($logger) {
                $logger->error($this->message, $this->context);
            }
        }
    }
}
