<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Providers;

use App\Services\Kafka\KafkaService;
use Illuminate\Support\ServiceProvider;
use Laravel\Lumen\Application as LumenApplication;
use longlang\phpkafka\Broker;
use longlang\phpkafka\Consumer\ConsumerConfig;
use longlang\phpkafka\Protocol\Metadata\MetadataRequest;

class KafkaServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        if( $this->app instanceof LumenApplication ){
            $this->app->configure('kafka');
        }




        $brokerClientId = env('KAFKA_HOST').":".env('KAFKA_PORT');

        $kafkaConfig = new ConsumerConfig();
        $kafkaBroker = (new Broker($kafkaConfig));
        $kafkaBroker->setBrokers([$brokerClientId]);

        try{
            $client = $kafkaBroker->getClient();
            if($client){
                $request = new MetadataRequest();
                $request->setTopics(['test2']);
                $request->setAllowAutoTopicCreation(true);
                $response = $client->sendRecv($request);
                var_dump($response->getTopics());
                exit;
            }
        }catch(\Exception $exception){
            var_dump($exception->getMessage());
            exit;
        }
    }
}
