<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Repositories;

use App\Exceptions\ServerRunTimeException;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Prettus\Repository\Eloquent\BaseRepository as BaseRepositoryEloquent;
use Prettus\Repository\Exceptions\RepositoryException;

abstract class BaseRepository extends BaseRepositoryEloquent
{
    public function listPageByPaginate($limit = null, $columns = ['*'], $paginateMethod = 'paginate'): array
    {
        $paginateResult = $this->paginate($limit,$columns,$paginateMethod);

        $paginate = [];
        $paginate['data'] = $paginateResult->items();
        $paginate['current_page'] = $paginateResult->currentPage();
        $paginate['total'] = $paginateResult->total();

        return $paginate;
    }

    public function transformData($datas, $modelName = null)
    {
        if(is_null($modelName)){
            return Arr::pluck($datas, null);
        }
        return app($modelName)->transform(Arr::pluck($datas, null));
    }

    public function getConnectionName()
    {
        return $this->makeModel()->getConnectionName();
    }

    public function getTableName()
    {
        return $this->makeModel()->getTable();
    }

    public function getConnection()
    {
        return DB::connection($this->getConnectionName());
    }

    public function createOrUpdate($params, $where = [], $autoUpdate = true)
    {
        $saveParams = $params;
        $modelPrimaryKey = $this->makeModel()->getKeyName();
        if(!empty($saveParams[$modelPrimaryKey])){
            $saveParams['updated_at'] = date('Y-m-d H:i:s',time());
        }else{
            $saveParams['created_at'] = date('Y-m-d H:i:s',time());
            $saveParams['updated_at'] = $saveParams['created_at'];
        }
        $attributes = array_keys($saveParams);

        $tableColumns = Schema::getColumnListing($this->makeModel()->getTable());
        foreach($attributes as $key => $attr){
            if(!in_array($attr, $tableColumns)){
                unset($attributes[$key]);
                unset($saveParams[$attr]);
            }
        }

        $updateWhere = $where;
        if(!empty($params[$modelPrimaryKey])){
            $updateWhere[$modelPrimaryKey] = $params[$modelPrimaryKey];
        }

        if(!$autoUpdate && !empty($where)){
            $findResult = $this->findWhere($updateWhere)->first();
            if($findResult){
                throw new RepositoryException('保存的数据已存在');
            }
        }

        try{
            if(empty($updateWhere)){
                return $this->create($saveParams);
            }
            return $this->updateOrCreate($updateWhere, $saveParams);
        }catch(\Throwable $exception){
            if($exception->getMessage() == ''){
                throw new ServerRunTimeException('保存失败');
            }
            throw new ServerRunTimeException($exception->getMessage());
        }
    }
}
