<?php

namespace App\Rhawn\Repositories\Eloquent;

use App\Rhawn\Repositories\Contracts\BhuaCustomerRepository;
use App\Rhawn\Repositories\Models\BhuaCustomer;
use Illuminate\Support\Facades\DB;
use Prettus\Validator\Contracts\ValidatorInterface;

/**
 * Class UserRepositoryEloquent.
 */
class BhuaCustomerRepositoryEloquent extends BaseRepository implements BhuaCustomerRepository
{
    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return BhuaCustomer::class;
    }

    public function getCustomerThroughtCusCode($cusCode)
    {
        $customer = $this->findWhere(['cus_no' => $cusCode])->first();
        if($customer){
            return $customer->toArray();
        }
        return null;
    }

    public function getCustomerList($where,$offset,$limit)
    {
        $customerModel = DB::connection($this->getConnectionName())->table($this->getTableName());
        if(!empty($where)){
            $customerModel = $customerModel->where($where);
        }
        $customerInfo = $customerModel->offset($offset)
            ->limit($limit)
            ->get();
        if($customerInfo){
            return $customerInfo->toArray();
        }
        return null;
    }

    /**
     * 获取客户折扣信息
     * @param $cusCode
     * @return array
     */
    public function getCustomerDiscountByCusCode($cusCode)
    {
        $customerDiscount = $this
        ->join('cusdiscount','cusdiscount.cus_id','customers.cus_id')
        ->where('customers.cus_no',$cusCode)
        ->get();

        $cusdiscountArr = [];
        if($customerDiscount){
           $customerDiscount = $customerDiscount->toArray();
            foreach($customerDiscount as $k=>$v){
                $cusdiscountArr[$v['b_id']]['cd_discount'] = $v['cd_discount'];
                $cusdiscountArr[$v['b_id']]['cd_pre_discount'] = $v['cd_pre_discount'];
            }
        }
        return $cusdiscountArr;
    }

    /**
     * 获取客户地址信息
     * @param $cusCode
     * @return array|null
     */
    public function getCustomerAddressByCusCode($cusCode)
    {
        $customerAddress = $this
            ->join('cusiaddrs','cusiaddrs.cus_id','customers.cus_id')
            ->where('customers.cus_no',$cusCode)
            ->orderBy('cusiaddrs.cia_if_default','DESC')
            ->orderBy('cusiaddrs.cia_id','DESC')
            ->get();

        if($customerAddress){
            return $customerAddress->toArray();
        }
        return null;
    }

    /**
     * 获取客户开票信息
     * @param $cusCode
     * @return array
     */
    public function getCustomerInvoiceByCusCode($cusCode)
    {
        $customerInvoice = $this
            ->join('cusinvoice','cusinvoice.cus_id','customers.cus_id')
            ->where('customers.cus_no',$cusCode)
            ->orderBy('cusinvoice.ci_if_default','DESC')
            ->orderBy('cusinvoice.ci_id','DESC')
            ->get();

        if($customerInvoice){
            $customerInvoice = $customerInvoice->toArray();
            foreach($customerInvoice as &$invoice){
                if($invoice['ci_type'] == 1){
                    $invoice['ci_type_name'] = '普票';
                }
                if($invoice['ci_type'] == 2){
                    $invoice['ci_type_name'] = '专票';
                }
            }
            return $customerInvoice;
        }
        return null;
    }
}
