<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Rhawn\Services;

use App\Rhawn\Repositories\Eloquent\RhawnSoitemsRepositoryEloquent;
use App\Rhawn\Repositories\Eloquent\RhawnSordersRepositoryEloquent;
use App\Services\PlatformDataEntriesService;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class ZhenkhService
{
    public function __construct()
    {
        $this->dataEntries = app(PlatformDataEntriesService::class);
        $this->rhawnOrdersRepository = app(RhawnSordersRepositoryEloquent::class);
        $this->rhawnSoitemsrepository = app(RhawnSoitemsRepositoryEloquent::class);
    }

    /**
     * @根据订单编号获取订单详情
     * @param $orderNo
     * @return array
     */
    public function getSorderDetail($orderNo)
    {
        $sOrder = $this->rhawnOrdersRepository->getSorderFromOrderNo($orderNo);
        if(!$sOrder){
            throw new \LogicException('该订单编号不存在!',500);
        }

        //查询订单的详情
        $soItems = $this->rhawnSoitemsrepository->getSorderItemsDetailFromOrderId($sOrder->so_id);
        //$soItems = $soItems->toArray();
        $returnItem = [];
        if($soItems){
            foreach($soItems as $item){
                $temp = [];
                $temp['b_cn_name'] = $item['b_cn_name'];
                $temp['p_code'] = $item['p_code'];
                $temp['c_cas'] = $item['c_cas'];
                $temp['p_cn_name'] = $item['p_cn_name'];
                $temp['si_price'] = $item['si_price'];
                $temp['si_discount'] = $item['si_discount'];
                $temp['si_p_tod'] = $item['si_p_tod'];
                $temp['si_amount'] = $item['si_amount'];
                $temp['si_vamount'] = $item['si_vamount'];
                $temp['si_note'] = $item['si_notes'];
                $temp['si_num'] = $item['si_num'];
                $temp['p_id'] = $item['p_id'];
                $temp['si_id'] = $item['si_id'];
                $temp['so_no'] = $sOrder->so_no;
                $temp['so_cus_po'] = $sOrder->so_cus_po;
                if($item['si_if_cancel'] == 0){
                    $returnItem[] = $temp;
                }
            }
        }

        return $returnItem;
    }

    /**
     * @订单发货单详情
     * @param $deliveryCode
     * @return false|mixed|null
     */
    public function getSorderDeliveryDetail($deliveryCode)
    {
        $dbConnect = DB::connection($this->rhawnOrdersRepository->getConnectionName());
        $dpdetail = $dbConnect->table('dispatch')
            ->join('dpdetail','dpdetail.dp_id','dispatch.dp_id')
            ->join('sorders','sorders.so_id','dispatch.so_id')
            ->join('soitems','soitems.si_id','dpdetail.si_id')
            ->where('dp_code',$deliveryCode)
            ->get()->toArray();
        if($dpdetail){
            return $dpdetail;
        }
        return null;
    }

    /**
     * @更新订单数据
     * @param $order
     * @param $orderDetail
     */
    public function updateZhenkhOrders($order,$orderDetail)
    {
        if(empty($orderDetail)){
            SimpleLogs::writeLog('order detail is empty . order_id :'.$order['purchaseOrderId'],__CLASS__.':updateZhenkhOrders');
            return false;
        }

        $dbConnect = DB::connection($this->rhawnOrdersRepository->getConnectionName());
        //判断订单是否已经存在
        $oneOrder = $dbConnect->table('sorders')
            ->where('so_cus_po',$order['purchaseOrderId'])
            ->get()->toArray();
        if($oneOrder){
            SimpleLogs::writeLog('order is exist . order_id :'.$order['purchaseOrderId'],__CLASS__.':updateZhenkhOrders', 'error');
            return false;
        }

        //获取订单发票信息
        $dataEntries = app(\App\Services\ThirdPlatform\ZhenKhService::class)->apiService->getPlatformDataEntries('zkh_order_invoice','data_values');

        try{
            $updateOrderArray = [];
            $updateOrderArray['cus_id'] = $dataEntries['id'];
            $updateOrderArray['so_no'] = 0;
            $updateOrderArray['so_cus_po'] = $orderDetail['purchaseOrderId'];
            $updateOrderArray['so_ca_name'] = $orderDetail['consigneeName'];
            $updateOrderArray['so_ca_mobile'] = $orderDetail['consigneeTel'];
            $updateOrderArray['so_ca_phone'] = '';
            list(
                $updateOrderArray['so_ca_province'],
                $updateOrderArray['so_ca_city'],
                $updateOrderArray['so_ca_county'],
                $updateOrderArray['so_ca_street']
                ) = explode(' ', $orderDetail['address']);
            $updateOrderArray['so_cia_name'] = $dataEntries['name'];
            $updateOrderArray['so_cia_mobile'] = $dataEntries['mobile'];
            $updateOrderArray['so_cia_phone'] = $dataEntries['phone'];
            $updateOrderArray['so_cia_province'] = $dataEntries['province'];
            $updateOrderArray['so_cia_city'] = $dataEntries['city'];
            $updateOrderArray['so_cia_county'] = $dataEntries['county'];
            $updateOrderArray['so_cia_street'] = $dataEntries['street'];
            $updateOrderArray['so_ci_type'] = $dataEntries['type'];
            $updateOrderArray['so_ci_title'] = $dataEntries['title'];
            $updateOrderArray['so_ci_vatno'] = $dataEntries['vatno'];
            $updateOrderArray['so_ci_bank'] = $dataEntries['bank'];
            $updateOrderArray['so_ci_bknum'] = $dataEntries['banknum'];
            $updateOrderArray['so_ci_addr'] = $dataEntries['address'];
            $updateOrderArray['so_ci_phone'] = $dataEntries['company_phone'];

            if($orderDetail['remark']){
                $updateOrderArray['so_note'] = $orderDetail['remark'];
            }else{
                $updateOrderArray['so_note'] = '';
            }
            if($order['isDirect']){
                $updateOrderArray['so_send_note'] = '发京东要求回单,发货单放箱子外面';
            }else{
                $updateOrderArray['so_send_note'] = '';
            }
            $updateOrderArray['so_standard'] = '';
            $updateOrderArray['so_if_wx'] = 0;
            $updateOrderArray['so_if_api'] = 0;
            $updateOrderArray['so_ci_method'] = 1;
            $updateOrderArray['so_total'] = $orderDetail['totalAmountFax'];
            $updateOrderArray['so_ctime'] = time();
            $updateOrderArray['so_creater'] = 29;

            $dbConnect->beginTransaction();

            $orderId = $dbConnect->table('sorders')->insertGetId($updateOrderArray);
            if($orderId > 0){

                $so_no = 1000000 + $orderId;
                $so_no = date('Ymd',time()).'-'.$so_no;
                $dbConnect->table('sorders')
                    ->where('so_id',$orderId)
                    ->update(['so_no' => $so_no]);

                $data['express'] = $orderDetail['totalFreight'];
                $tempSku = [];
                foreach($orderDetail['lines'] as $line){
                    $tempSku[] = $line['zkhSku'];
                    if(!empty($tempSku)) {
                        //获取对应关系
                        $goods = $dbConnect->table('zkh_goods')
                            ->where('zg_sku', $line['zkhSku'])
                            ->get()->toArray();
                        if (count($goods) > 0) {
                            $pIds = [];
                            foreach ($goods as $goodsInfo) {
                                $pIds[] = $goodsInfo->zg_r_pid;
                            }
                            if (!empty($pIds)) {
                                $products = $dbConnect->table('products')
                                    ->whereIn('p_id', $pIds)
                                    ->get()->toArray();
                                if ($products) {
                                    foreach ($products as $key => $product) {
                                        $orderItemsArray = [];
                                        $orderItemsArray['so_id'] = $orderId;
                                        $orderItemsArray['p_id'] = $product->p_id;
                                        $orderItemsArray['si_price'] = $product->p_price;
                                        $orderItemsArray['si_discount'] = $line['priceFax'];
                                        $orderItemsArray['si_num'] = $line['qty'];
                                        $orderItemsArray['si_amount'] = $line['amountFax'];
                                        $orderItemsArray['si_p_tod'] = $product->p_tod;
                                        $orderItemsArray['si_vamount'] = 0;
                                        $orderItemsArray['si_nodes'] = '';
                                        $orderItemsArray['si_notes'] = '';
                                        $orderItemsArray['si_sales_note'] = '';
                                        $orderItemsArray['si_cus_note'] = '';

                                        $dbConnect->table('soitems')->insert($orderItemsArray);
                                    }
                                }
                            }
                        }
                    }
                }
                if($data['express'] > 0){
                    $soitems = [];
                    $soitems_count = count($products);
                    $soitems[$soitems_count]['so_id'] = $orderId;
                    $soitems[$soitems_count]['p_id'] = 0;
                    $soitems[$soitems_count]['si_price'] = 0;
                    $soitems[$soitems_count]['si_discount'] = 0;
                    $soitems[$soitems_count]['si_num'] = 0;
                    $soitems[$soitems_count]['si_amount'] = $data['express'];
                    $soitems[$soitems_count]['si_vamount'] = 0;
                    $soitems[$soitems_count]['si_p_tod'] = '';
                    $soitems[$soitems_count]['si_cus_note'] = '';
                    $soitems[$soitems_count]['si_molbase_pog'] = '';

                    $dbConnect->table('soitems')->insert($soitems);
                }
            }
            $dbConnect->commit();
            return true;
        }catch(\Throwable $exception){
            $dbConnect->rollBack();
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.":updateZhenkhOrders", 'error');
            throw new \Exception($exception->getMessage(),'502');
        }
    }

    /**
     * 保存震坤行同步商品数据
     * @param $goods
     */
    public function updateZhenkhGoods($goods)
    {
        try{
            $dbConnect = DB::connection('rhawn_mysql');

            $zkhGoods = [];
            $zkhGoods['zg_sku'] = $goods['zkhSku'];
            $zkhGoods['zg_r_sku'] = $goods['supplierSkuNo'];
            if(isset($goods['addressList']['0'])){
                $zkhGoods['zg_addr'] = implode(',',$goods['addressList']);
            }
            $zkhGoods['zg_unit'] = $goods['priceUnit'];

            if($zkhGoods['zg_r_sku'] != ''){
                $zkhGoodsResult = $dbConnect->table('products')
                    ->where('p_code',$zkhGoods['zg_r_sku'])
                    ->get()->toArray();
                if($zkhGoodsResult){
                    $zkhGoodsResult = current($zkhGoodsResult);
                    $zkhGoods['zg_r_pid'] = $zkhGoodsResult->p_id;


                    $row = $dbConnect->table('zkh_goods')
                        ->where('zg_r_sku',$zkhGoods['zg_r_sku'])
                        ->where('zg_r_pid',$zkhGoods['zg_r_pid'])
                        ->get()->toArray();

                    if(count($row) == 0){
                        $id = $dbConnect->table('zkh_goods')->insertGetId($zkhGoods);
                        SimpleLogs::writeLog('zkh_goods 新增 id : '.$id.' 成功.', 'sql insert');
                        $row = $dbConnect->table('zkh_goods')->where('zg_id',$id)->get()->toArray();
                    }
                    if($row){
                        return current($row);
                    }
                }
            }
            return null;

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),'updateZhenkhGoods ','error');
            throw $exception;
        }
    }

    /**
     * 更新震坤行数据更新时间
     * @param $zgId
     * @return int
     */
    public function updateSyncRunTime($zgId)
    {
        $dbConnect = DB::connection('rhawn_mysql');
        return $dbConnect->table('zkh_goods')->where('zg_id',$zgId)->update(['zg_sync_time' => time()]);
    }

    /**
     * @获取震坤行对应的商品sku
     * @param $pCode
     * @return array
     */
    public function getZkhGoodsSku($pCode)
    {
        $dbConnect = DB::connection('rhawn_mysql');
        $model = $dbConnect->table('zkh_goods');
        if(is_array($pCode)){
            $model->whereIn('zg_r_pid',$pCode);
        }else{
            $model->where('zg_r_pid',$pCode);
        }
        return $model->get()->toArray();
    }

}