<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services\Api;

use App\Finance\Services\InvoiceService;
use App\Repositories\Transformers\Rhawn\InvoiceListTransformer;
use App\Repositories\Transformers\Rhawn\InvoiceSorderItemsTransformer;
use App\Rhawn\Repositories\Eloquent\BhuaInvoiceRepositoryEloquent;
use App\Rhawn\Repositories\Eloquent\RhawnInvoiceRepositoryEloquent;
use Illuminate\Support\Facades\DB;

class InvoicesService
{
    public function __construct(InvoiceService $invoiceService)
    {
        $this->rhawnOrdersService = app(\App\Rhawn\Services\RhawnOrdersService::class);
        $this->invoiceService = $invoiceService;
        $this->invoiceRepository = app(RhawnInvoiceRepositoryEloquent::class);
        $this->bHuaInvoiceRepository = app(BhuaInvoiceRepositoryEloquent::class);
    }

    public function getInoviceItems($requestParams)
    {
        if(!isset($requestParams['invoice_number']) || empty($requestParams['invoice_number'])){
            throw new \Exception('发票号码为空',502);
        }

        $invoiceRecord = $this->invoiceService->invoiceRecordRepository->getInvoiceRecordByRealNumber($requestParams['invoice_number']);
        if(!$invoiceRecord){
            throw new \Exception('该发票号码无效',502);
        }
        $invoiceRecord = current($invoiceRecord);

        switch($invoiceRecord['invoice_platform']){
            case 'rhawn' : $invoiceItems = $this->invoiceRepository->getInvoiceItemsByInvoiceNumber($invoiceRecord['invoice_real_number']);break;
            case 'bhua' : $invoiceItems = $this->bHuaInvoiceRepository->getInvoiceItemsByInvoiceNumber($invoiceRecord['invoice_real_number']);break;
        }
        if(!$invoiceItems){
            throw new \Exception('无法查询该发票号对应明细',502);
        }
        $invoiceItemsId = [];
        foreach($invoiceItems as $item){
            $invoiceItemsId[] = $item['soid_id'];
        }

        switch($invoiceRecord['invoice_platform']){
            case 'rhawn' : $sorderItems = $this->invoiceService->rhawnInvoiceItemsRepository->getInvoiceSorderItemsBySoItemsId($invoiceItemsId);break;
            case 'bhua' : $sorderItems = $this->invoiceService->bhuaInvoiceItemsRepository->getInvoiceSorderItemsBySoItemsId($invoiceItemsId);break;
        }

        if(!$sorderItems){
            throw new \Exception('查询对应订单错误',502);
        }

        $sorderItems = app(InvoiceSorderItemsTransformer::class)->transform($sorderItems);
        return $sorderItems;
    }

    public function getInoviceListByOrderNumber($requestParams)
    {
        if(!isset($requestParams['order_number']) || empty($requestParams['order_number'])){
            throw new \Exception('订单编号为空',502);
        }

        $orderNumber =  $requestParams['order_number'];
        $dpdetailList = $this->rhawnOrdersService->getOrderDispatchDetail($orderNumber);
        if(!$dpdetailList){
            return null;
        }

        $dpdIds = array_column($dpdetailList, 'dpd_id');
        $invoiceList = $this->invoiceRepository->getInvoiceItemsByDpdIds($dpdIds);

        if($invoiceList){
            $invoiceList = app(InvoiceListTransformer::class)->transform($invoiceList);
        }

        return $invoiceList;
    }

}
