<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Repositories\Contracts\PlatformCustomerRepository;
use App\Rhawn\Services\RhawnCustomerService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\Helpers;
use Illuminate\Support\Facades\DB;

class AuthService
{
    use Helpers;

    public function __construct(PlatformCustomerRepository $platformCustomerRepository)
    {
        $this->platformCustomerRepository = $platformCustomerRepository;
    }

    /**
     * 获取token
     * @param $requestParams
     * @return array
     * @throws \Exception
     */
    public function getToken($requestParams)
    {
        $token = $requestParams['customer_token'];

        $platformCustomer = $this->platformCustomerRepository->findWhere([
            'token' => $token
        ]);
        if(!$platformCustomer->toArray()){
            throw new \Exception('token无效',500);
        }

        $expTimestamp = strtotime(date('Y-m-d H:i:s', strtotime('+1 day')));
        $jwtToken = $this->generateToken($token);

        return [
            'token' => $jwtToken,
            'ttl' => $expTimestamp
        ];
    }

}
