<?php

namespace App\Services\Kafka;

use App\Repositories\Contracts\KafkaFailQueueRepository;
use App\Services\SysLogService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\KafkaConfigHelpers;
use longlang\phpkafka\Consumer\ConsumeMessage;
use longlang\phpkafka\Consumer\Consumer;
use longlang\phpkafka\Producer\Producer;


class KafkaService
{
    use KafkaConfigHelpers;

    protected $kafkaProducerConfig;
    protected $kafkaConsumerConfig;
    protected $config = null;

    public function __construct($config)
    {
        $this->kafkaProducerConfig = $this->getProducerConfig($config);
        $this->config = $config;
    }

    /**
     * 推送记录
     * @param $contentValue
     * @param null $topic
     */
    public function produerSend($contentValue, $topic = null)
    {
        try {
            $kafkaTopic = $topic;
            if ($kafkaTopic == null) {
                $kafkaTopic = $this->config['topic'];
            }else{
                $this->config['topic'] = $topic;
            }
            $key = uniqid($kafkaTopic, true);
            $producer = new Producer($this->kafkaProducerConfig);
            $producer->send($kafkaTopic, json_encode($contentValue), $key);

            SimpleLogs::writeLog([
                'content' => $contentValue
            ],__CLASS__.':produerSend');

        } catch (\Exception $exception) {
            $this->errLog($exception,__CLASS__.":produerSend , topic is ".$kafkaTopic);
            throw $exception;
        }
    }

    /**
     * 开启消费者
     */
    public function startConsumer($topic = null)
    {
        if(is_null($topic)){
            $this->config['topic'] = $topic;
        }
        $this->kafkaConsumerConfig = $this->getConsumerConfig($this->config);
        $consumer = new Consumer($this->kafkaConsumerConfig, function (ConsumeMessage $message) {

            try{
                $consumer = $message->getConsumer();
                $kafkaMessage = json_decode($message->getValue(),true);
                $consumer->ack($message); // 手动提交
                app($kafkaMessage['consumer'])->{$kafkaMessage['method']}($kafkaMessage['params']);
            }catch(\Throwable $exception){
                $this->errLog($exception,$this->kafkaConsumerConfig->getGroupInstanceId().'_consumer');
            }

            $consumer->stop();
        });
        $consumer->start();
        $consumer->close();
    }

    /**
     * 震坤行消费
     */
    public function startZkhConsumer()
    {
        $this->config['topic'] = 'zkh';
        $this->kafkaConsumerConfig = $this->getConsumerConfig($this->config);
        $consumer = new Consumer($this->kafkaConsumerConfig, function (ConsumeMessage $message) {
            try{
                $consumer = $message->getConsumer();
                $kafkaMessage = json_decode($message->getValue(),true);
                $consumer->ack($message); // 手动提交
                app($kafkaMessage['consumer'])->{$kafkaMessage['method']}($kafkaMessage['params']);
            }catch(\Throwable $exception){
                $this->errLog($exception,$this->kafkaConsumerConfig->getGroupInstanceId().'_consumer');
            }
            $consumer->stop();
        });
        $consumer->start();
        $consumer->close();
    }

    protected function errLog($exception, $errorType)
    {
        SimpleLogs::writeLog([
            'error' => $exception->getCode(),
            'message' => $exception->getMessage()
        ], $errorType, 'error');
    }

    public function storeFailQueue($failQueueContent)
    {
        $repository = app(KafkaFailQueueRepository::class);
        foreach($failQueueContent as $key => &$content){
            if(is_array($content) || $content instanceof \ArrayObject){
                $content = json_encode($content);
            }
        }
        $repository->create($failQueueContent);

    }

}
