<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Repositories\Contracts\SysLogRepository;
use App\Repositories\Enums\LogEnum;
use Monolog\Logger;
use Monolog\Processor\WebProcessor;
use Psr\Log\LoggerInterface;

class SysLogService
{
    public function addOperationLogs($logsContent)
    {
        $context = [
            'content' => $logsContent,
            'optime' => microtime(true),
        ];
        logger_async(LogEnum::SYSTEM_OPERATION, $context)
            ->onConnection(\config('logging.request.connection'))
            ->onQueue(\config('logging.request.queue'));
    }

    public function addApiLogs($logsContent)
    {
        $context = [
            'content' => $logsContent,
            'optime' => microtime(true),
        ];
        logger_async(LogEnum::SYSTEM_API, $context)
            ->onConnection(\config('logging.request.connection'))
            ->onQueue(\config('logging.request.queue'));
    }

    public function writeLog($content, $logType, $level = 'info')
    {
        $context = [
            'content' => is_array($content) || is_object($content) ? json_encode($content,JSON_UNESCAPED_UNICODE) : $content,
            'optime' => microtime(true),
        ];

        $this->write($level, $logType.':', $context, false);
    }

    public function httpLog()
    {

    }

    protected function write($type, $message, $context, $http = false)
    {
        app()->forgetInstance(LoggerInterface::class);
        // unset(app()[LoggerInterface::class]);

        $logger = app(LoggerInterface::class)->getLogger();
        if ($logger instanceof Logger && $http) {
            $logger->pushProcessor(new WebProcessor($this->serverData));
        }

        switch($type){
            case 'info' : $logger->info($message, $context);break;
            case 'debug' : $logger->debug($message, $context);break;
            case 'error' : $logger->error($message, $context);break;
        }
    }

}
