<?php

namespace App\Services\ThirdPlatform\Api;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\HttpClientHelpers;


class CheMenuApiService extends PlatformAbstractService
{
    use HttpClientHelpers;

    private $yybaoApiUrl = 'http://localhost:8216';

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('chemenu');
        $this->getPlatformInfo();
    }

    public function getAllCatetgory()
    {
        try {
            if ($this->checkPlatformStatus()) {
                return $this->clientRequest('get', $this->getDomain() . '/category/New.html', [], false);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    public function saveCategory($categoryName, $parentCategoryName, $webPath)
    {
        try{
            $response = $this->getPostClient($this->yybaoApiUrl.'/webpage/chemenu/saveCategory',
                [
                    'categoryName' => $categoryName,
                    'parentName' => $parentCategoryName,
                    'webPath' => $webPath
                ], []);
            return $this->returnResponse($response);
        }catch(\Throwable $exception){
            if(preg_match('/\{(?<response>[\s\S]*?)\}/i', $exception->getMessage(), $response)){
                $response = json_decode('{'.$response['response'].'}', true);
                if($response && $response['status'] == 429){
                    sleep(60);
                    $this->saveCategory($categoryName, $parentCategoryName, $webPath);
                }
            }
            $this->requestError($exception);
        }
    }

    public function saveProduct($productContent)
    {
        try{
            $response = $this->getPostClient($this->yybaoApiUrl.'/webpage/chemenu/saveProduct',
                [
                    'productName' => $productContent['productName'],
                    'iupac' => $productContent['IUPAC'],
                    'image' => $productContent['image'],
                    'casInfo' => json_encode($productContent['casInfo']),
                    'productList' => json_encode($productContent['productList'])
                ], []);
            return $this->returnResponse($response);
        }catch(\Throwable $exception){
            dd($exception->getMessage());
            if(preg_match('/\{(?<response>[\s\S]*?)\}/i', $exception->getMessage(), $response)){
                $response = json_decode('{'.$response['response'].'}', true);
                if($response && $response['status'] == 429){
                    sleep(60);
                    $this->saveProduct($productContent);
                }
            }
            $this->requestError($exception);
        }
    }


    public function returnResponse($apiResponse): bool
    {
        $response = json_decode($apiResponse, true);
        if($response){
            if($response['status'] == 'success' && $response['code'] == 200){
                return true;
            }
        }
        return false;
    }

    protected function requestError($response)
    {
        SimpleLogs::writeLog($response->getMessage(), 'chemenu error', 'error');
        throw $response;
    }
}
