<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services\ThirdPlatform\Api;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Traits\HttpClientHelpers;


class IntegleApiService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('integle');
        $this->getPlatformInfo();
    }

    /**
     * 批量更新商品信息
     * @param $product
     */
    public function pushBatchUpdateProduct($product)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['ak'] = $this->platformInfo['platform_params']['ak'];
                $params['product'] = $this->mergeApiMetaData($product);
                $response = $this->getPostClient('http://suppliers.integle.com/openapi/chem/batch-save',$params);

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            return $this->requestError($e);
        }
    }

    /**
     * 整理组装 api 数据
     * @param $metaData
     * @return array
     */
    public function mergeApiMetaData($metaData)
    {
        if(empty($metaData)){
           return [];
        }

        $mergeData = [];
        foreach($metaData['rawList'] as $datas){

            $tempData['in_product_id'] = $datas['r_code'];
            $tempData['cas'] = $datas['c_cas'];
            $tempData['cn_name'] = $datas['r_cn_name'];
            $tempData['en_name'] = $datas['r_en_name'];
            $tempData['brand'] = '罗恩';
            $tempData['package'] = [];

            if(isset($metaData['packageList'][$datas['r_code']])){
                $package = $metaData['packageList'][$datas['r_code']];
                $pack['in_product_id'] = $package['p_code'];
                $pack['purity'] = $package['r_level'];
                $pack['package_size'] = $package['p_pack'];
                $pack['package_unit'] = $package['p_pack_unit'];
                $pack['price'] = $package['p_price'];
                $pack['ship_within_time'] = 0;
                $pack['area_stock'] = ['location' => '上海', 'inventory' => $package['p_stock']];
                $pack['promotion_tag'] = '';
                $pack['status'] = $package['p_status'] == '1' ? 1 : 0;

                array_push($tempData['package'],$pack);
            }

            array_push($mergeData,$tempData);
            unset($tempData);
        }

        return $mergeData;
    }

    protected function apiResponse($response)
    {
        $returnResult = [];

        $decodeResponse = json_decode($response,true);
        $returnResult['resultContent'] = $decodeResponse;
        $returnResult['status'] = false;

        if($decodeResponse){
            if($decodeResponse['status'] == 1){
                $returnResult['status'] = true;
            }
        }
        return $returnResult;
    }

    protected function requestError($response)
    {
       return [
           'status' => false,
           'resultContent' => [
               'error_code' => $response->getCode(),
               'error_message' => $response->getMessage()
           ],
       ];
    }
}
