<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services\ThirdPlatform\Api;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\HttpClientHelpers;


class LeyanApiService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('leyan');
        $this->getPlatformInfo();
    }

    /**
     * 批量更新商品信息
     * @param $product
     * @return bool
     */
    public function pushUpdateProductByCas($cas,$token)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['casNo'] = $cas;

                $uri = $this->platformInfo['platform_url'].'/ProductDate/findProductDataPoolByCatalogNoOrCas';
                $response = $this->getPostClient($uri,$params,null,[
                    'token' => $token,
                ],false);

                return $this->apiResponse($response);
            }
        }catch(\exception $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':pushUpdateProductByCas error', 'error');
            return false;
        }
    }

    public function pushBatchUpdateProductByPage($currentPage, $pageSize = 1000, $token)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['nowPage'] = $currentPage;
                $params['pageSize'] = $pageSize;

                $uri = $this->platformInfo['platform_url'].'/ProductDate/findProductDataPoolPage';
                $response = $this->getPostClient($uri,$params,null,[
                    'token' => $token,
                ],false);

                return $this->apiResponse($response);
            }
        }catch(\exception $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':pushBatchUpdateProduct error', 'error');
            return false;
        }
    }

    public function getToken(){
        try{
            if($this->checkPlatformStatus()){
                $response = $this->clientRequest('get',
                    $this->platformInfo['platform_url'].'/ProductDate/getToken?username='.$this->platformInfo['platform_params']['username'].'&password='.$this->platformInfo['platform_params']['password'],[]
                );
                //$token = $this->apiResponse($response);
                if(!empty($response)){
                    return $response;
                }
            }
            return null;

        }catch(\Exception $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':token error', 'error');
            return false;
        }
    }

    /**
     * 整理组装 api 数据
     * @param $metaData
     * @return array
     */
    public function mergeApiMetaData($metaData)
    {
        return $metaData;
    }

    protected function apiResponse($response)
    {
        $decodeResponse = json_decode($response,true);
        return $decodeResponse;
        if($decodeResponse['status'] == '1' && $decodeResponse['code'] == '0'){
            if(!empty($decodeResponse['data'])){
                return $decodeResponse['data'];
            }
        }elseif($decodeResponse['status'] == '0'){
            if(!is_array($decodeResponse['data'])){
                return false;
            }
        }
        return false;
    }
}
