<?php

namespace App\Services\ThirdPlatform\Api;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\HttpClientHelpers;


class WuxiApiNewService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('wuxi_appTec');
        $this->getPlatformInfo();
    }

    public function getToken()
    {
        return $this->postApi('/token/fetch', [
            'username' => '790087059@qq.com',
            'password' => 'YXH1992??!!'
        ]);
    }

    /**
     * 批量更新商品信息
     * @param $product
     * @return bool
     */
    public function pushBatchUpdateProduct($product, $token)
    {
        $params = $this->mergeApiMetaData($product);
        return $this->postApi('/api/v2/updateChamical', $params, $token);
    }

    /**
     * @更新包装规格
     * @param $packages
     * @param $token
     * @return false|mixed|void
     */
    public function pushBatchUpdatePackages($updateData,$token)
    {
        $packageList = [];
        foreach($updateData as $packageCode => $data){
            $tempData = [];
            $tempData['product_division'] = 'CHEMICAL';
            $tempData['productId'] = $data['r_code'];
            $tempData['status'] = $data['p_status'] == '1' ? 'ACTIVE' : 'INACTIVE';
            $tempData['packageUnitDescription'] = $packageCode;
            $packageList[] = $tempData;
        }
        return $this->postApi('/api/v2/syncProductPackageStatus', $packageList, $token);
    }

    public function pushBatchUpdatePrice($updateData,$token)
    {
        $updateProductPrice = [];
        foreach($updateData as $product){
            $tempData = [];
            $tempData['product_division'] = 'CHEMICAL';
            $tempData['productId'] = $product['r_code'];
            $tempData['packageUnitDescription'] = $product['p_code'];
            $tempData['minPackageUnitDescription'] = '';
            $tempData['price'] = bcmul($product['p_price'], 1, 2);
            $tempData['discount'] = '0.5';
            $tempData['inCostCurrency'] = 'RMB';
            $tempData['status'] = $product['p_status'] == '1' ? 'ACTIVE' : 'INACTIVE';;
            $updateProductPrice[] = $tempData;
        }
        return $this->postApi('/api/v2/syncPackagePrice', $updateProductPrice, $token);
    }

    public function pushBatchUpdateStock($updateData,$token)
    {
        $updateProductStock = [];
        foreach($updateData as $product){
            $tempData = [];
            $tempData['product_division'] = 'CHEMICAL';
            $tempData['productId'] = $product['r_code'];
            $tempData['packageUnitDescription'] = $product['p_code'];
            $tempData['packageStockQuantity'] = $product['p_stock'];
            $tempData['packageStockLocation'] = 'CN-SH';
            $tempData['stockStatus'] = $product['p_status'] == '1' ? 'ACTIVE' : 'INACTIVE';;
            $updateProductStock[] = $tempData;
        }
        return $this->postApi('/api/v2/syncRealTimeInventory', $updateProductStock, $token);
    }

    /**
     * 整理组装 api 数据
     * @param $metaData
     * @return array
     */
    public function mergeApiMetaData($metaData)
    {
        if(empty($metaData)){
            return [];
        }

        $mergeData = [];
        foreach($metaData['rawList'] as $datas){

            if(!empty($datas['product_extend'])){
               $tempData['supplier_product_id'] = $datas['r_code'];
               $tempData['mdl'] = $datas['product_extend']['mdl'] ?? '';
               $tempData['cas'] = $datas['c_cas'];
               $tempData['nu'] = $datas['product_extend']['un'] ?? '';
               $tempData['chinese_name'] = empty($datas['product_extend']['chinese_name']) ? $datas['r_cn_name'] : $datas['product_extend']['chinese_name'];
               $tempData['english_name'] = empty($datas['product_extend']['english_name']) ? $datas['r_en_name'] : $datas['product_extend']['english_name'];
               $tempData['chemicalformula'] = empty($datas['product_extend']['molecular_formula']) ? $datas['c_fzs'] : $datas['product_extend']['molecular_formula'];
               $tempData['molecular_weight'] = $datas['product_extend']['molecular_weight'] ?? '';
               $tempData['brand'] = $datas['product_extend']['brand'] ?? '';
               $tempData['purity'] = $datas['product_extend']['purity'] ?? '';

               $tempData['usually_ships_days_min'] = $datas['product_extend']['lead_time_min_days'] ?? '';
               $tempData['usually_ships_within_days'] = $datas['product_extend']['lead_time_max_days'] ?? '';
               $tempData['backorder_lead_time'] = $datas['product_extend']['lead_time_back_order_days'] ?? '';
                //储存条件
               $tempData['storage_condition'] = $datas['product_extend']['storage_condition'];
                //运输条件
               $tempData['transportation_condition'] = $datas['product_extend']['transportation_condition'];
               $tempData['smiles'] = $datas['c_smiles'];
               $tempData['sub_category'] = empty($datas['product_extend']['sub_category']) ? 'PURE_COMPOUNDS_FOR_INDUSTRIAL_USE' : $datas['product_extend']['sub_category'];

               $tempData['group_id'] = 6;//$datas['product_extend']['product_category'];
                //是否进口
               $tempData['is_imported'] = $datas['product_extend']['is_imported'];
                //是否推送电商
               $tempData['is_push_ds'] = false;

               $tempData['ships_additional_description'] = $datas['product_extend']['ships_additional_description'];
               $tempData['chemical_dissolve_cas'] = $datas['product_extend']['chemical_dissolve_cas'];
            }
            /*$tempData['supplier_product_id'] = $datas['r_code'];
            $tempData['mdl'] = '';
            $tempData['cas'] = $datas['c_cas'];
            $tempData['nu'] = '';
            $tempData['chinese_name'] = $datas['r_cn_name'];
            $tempData['english_name'] = $datas['r_en_name'];
            $tempData['chemicalformula'] = $datas['c_fzs'] ?? '';
            $tempData['purity'] = '00AR';//$datas['r_level'];

            /*$tempData['usually_ships_days_min'] = '';
            $tempData['usually_ships_within_days'] = '';
            $tempData['backorder_lead_time'] = '';*/
            //储存条件
            /*$tempData['storage_condition'] = 0;
            switch($datas['c_cctj']){
                case 1:  $tempData['storage_condition'] = 0;break;
                case 2:  $tempData['storage_condition'] = 1;break;
                case 3:  $tempData['storage_condition'] = 2;break;
            }

            //运输条件
            switch ($tempData['storage_condition']){
                case 0: $tempData['transportation_condition'] = 0;break;
                case 1: $tempData['transportation_condition'] = 2;break;
                case 2: $tempData['transportation_condition'] = 1;break;
            }

            $tempData['smiles'] = $datas['c_smiles'];
            $tempData['sub_category'] = 'INHIBITOR';

            /*产品种类
            1: Templates/Scaffolds，
2: Amino Acids &
Peptides，
3：Building Blocks，
4：Screening Library
Compounds，
5：Fragment Library
Compounds，
6：Reagent Services，
7：Natural Products，
8：Impurity
            */
            /*$tempData['group_id'] = 6;
            //是否进口
            $tempData['is_imported'] = false;
            //是否推送电商
            $tempData['is_push_ds'] = false;

            //$tempData['status'] = $datas['r_status'] == '1' ? 'ACTIVE' : 'INACTIVE';

            /*$tempData['inventorys'][] = [
                'type' => $datas['r_unit'],
                'quantity' => floatval($datas['r_stock']),
                'country' => 'CN',
                'stock_status' => $datas['r_stock'] > 0 ? 'INSTOCK' : 'OUTSTOCK',
            ];*/

            $tempData['packages'] = [];
            $currentPackageList = $metaData['packageList'];
            if(!empty($currentPackageList)){
                foreach($currentPackageList as $package){
                    $pack = [];
                    if($package['r_code'] == $datas['r_code']){
                        $package['p_price'] = ceil($package['p_price']);
                        $pack['unit_description'] = $package['p_pack'].''.$package['p_pack_unit'];
                        //$pack['package_status'] = $package['p_status'] == '1' && $package['p_show'] == '1' && $package['p_if_big'] == 0 ? 'ACTIVE' : 'INACTIVE';
                        //$tempData['status'] = $package['p_status'] == '1' && $package['p_show'] == '1' && $package['p_if_big'] == 0 ? 'ACTIVE' : 'INACTIVE';

                        $pack['inventorys'][] = [
                            'package_quantity' => '1',//$package['p_stock'],
                            //'stock_status' => $package['p_stock'] > 0 ? 'INSTOCK' : 'OUTSTOK',
                            'country' => 'CN-SH'
                        ];

                        $pack['prices'][] = [
                            'price' => bcmul($package['p_price'], 1, 2),
                            'currency' => 'RMB',
                            //'status' => 'ACTIVE'
                        ];

                        /*$pack['costs'] = [
                            'In_cost' => $package['p_avg_cost'],
                            'In_cost_currency' => 'RMB',
                            'status' => $package['p_stock'] > 0 ? 'ACTIVE' : 'INACTVIE',
                        ];*/

                        $pack['wuxiPackagePricings'][] = [
                            'discount'=> 0.5,
                            'currency' => 'RMB',
                            //'status' => 'INACTIVE'
                        ];
                        $tempData['packages'][] = $pack;
                    }
                }
            }
            $mergeData[] = $tempData;
            unset($tempData);
        }

        return $mergeData;
    }

    protected function postApi($apiUri, $postParams, $token = null)
    {
        try {
            if ($this->checkPlatformStatus()) {
                $response = $this->getPostClient($this->platformInfo['platform_url'] . $apiUri, $postParams, null,
                    !is_null($token) ? [ 'Authorization' => 'Bearer ' . $token] : null
                );
                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    protected function apiResponse($response)
    {
        $decodeResponse = json_decode($response,true);
        if($decodeResponse){
            return $decodeResponse;
        }
        return false;
    }

    protected function requestError($response)
    {
        SimpleLogs::writeLog($response->getMessage(), 'WuxiApi error', 'error');
        throw $response;
    }
}
