<?php

namespace App\Services\ThirdPlatform;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\Api\CheMenuApiService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\Helpers;

class CheMenuService
{
    use Helpers;

    public function __construct()
    {
        $this->apiService = (new CheMenuApiService(app(ThirdApiPlatformRepository::class)));
    }

    public function allCategory()
    {
        $responseHtml = $this->apiService->getAllCatetgory();
        if($responseHtml){
            $categoryDivRegx = '/<dt><span[\s]*?style=[\\\'|\"][\s\S]+?[\\\'|\"]>(?<categoryH1>[\s\S]+?)<\/span>[\s\S]*?<dd>(?<category>[\s\S]+?)<\/dd>/i';
            preg_match_all($categoryDivRegx, $responseHtml,$categoryHtml);
            if($categoryHtml){
                foreach($categoryHtml['categoryH1'] as $key => $h1){
                    if(!empty($categoryHtml['category'][$key])){
                        preg_match_all('/<a[\s]*?href=[\\\'|\"](?<href>[\s\S]+?)[\\\'|\"]>(?<categoryName>[\s\S]+?)<\/a>/i', $categoryHtml['category'][$key],$categoryList);
                        if(!empty($categoryList)){
                            foreach($categoryList['categoryName'] as $k => $category){
                                $this->apiService->saveCategory($category, $h1, $categoryList['href'][$k]);
                                $jobContent = [
                                    'params' => ['url' => $categoryList['href'][$k]],
                                    'consumer' => __CLASS__,
                                    'method' => 'categoryPage'
                                ];
                                $this->apiService->pushQueue($jobContent,'chemenu_category_page');
                            }
                        }
                    }
                    sleep(60);
                }
            }

            return true;
        }
    }

    public function categoryPage($url)
    {
        try{
            $pageUrl = $url['url'];
            while(true){
                $responseHtml = $this->apiService->getPage($pageUrl);
                //file_put_contents('./test2.txt', $responseHtml);
                if($responseHtml){
                    $titleDivRegx = '/<div[\s]*?class=[\\\'|\"]cate_desc[\\\'|\"]>[\S\s]*<h2[\s]*?class=[\\\'|\"]desc_title[\\\'|\"]>(?<title>[\s\S]+?)<\/h2>[\s\S]*?<p[\s]*?class=[\\\'|\"]desc_content[\\\'|\"]>(?<content>[\s\S]*?)<\/p>/i';
                    preg_match($titleDivRegx, $responseHtml,$titleHtml);

                    $productDivRegx = '/<div[\s]*?class=[\\\'|\"]col-lg-3 col-sm-6 col-md-4 col-xs-6 products_rows_div[\\\'|\"]>[\S\s]*?<div[\s]*?class=[\\\'|\"]cas_img[\\\'|\"]>[\s]*?<a[\s]*href=[\\\'|\"](?<href>[\S\s]+?)[\\\'|\"][\s\S]*?>[\s\S]+?<\/div>/i';
                    preg_match_all($productDivRegx, $responseHtml, $productList);
                    if($productList){
                        foreach($productList['href'] as $href){
                            $productJobContent = [
                                'params' => ['url' => $href],
                                'consumer' => __CLASS__,
                                'method' => 'productPage'
                            ];
                            $this->apiService->pushQueue($productJobContent,'chemenu_product_page');
                        }

                        //分页处理
                        if(preg_match('/<nav[\s]*?class=[\\\'|\"]paging[\\\'|\"]>/i', $responseHtml)){
                            if(preg_match('/<li[\s]*?id=[\\\'|\"]pagination_next[\\\'|\"][\s]*?class=[\\\'|\"]disabled[\\\'|\"]>/i', $responseHtml)){
                                break;
                            }
                            preg_match('/<li[\s]*?id=[\\\'|\"]pagination_next[\\\'|\"]>[\s]*?<a[\S\s]*?href=[\\\'|\"](?<nextHref>[\s\S]+?)[\\\'|\"]/i', $responseHtml, $nextHref);
                            $pageUrl = str_replace($this->apiService->getDomain(), '', $nextHref['nextHref']);
                        }
                    }
                }
            }
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), 'chemenu error', 'error');
        }
    }


    public function productPage($url)
    {
        try{
            $pageUrl = $url['url'];
            $responseHtml = $this->apiService->getPage($pageUrl);
            //file_put_contents('./test3.html', $responseHtml);
            if($responseHtml){
                $productPageContent = [];
                if(preg_match('/<div[\s]*?class=[\\\'|\"]products-content[\\\'|\"]>/i', $responseHtml)){
                    $imageDivRegx = '/<div[\s]*?class=[\\\'|\"]product-image[\\\'|\"]>[\s\S]*?<img[\s]*?class=[\\\'|\"]big_img[\\\'|\"][\s]*?src[\s\S]*?[\\\'|\"](?<image>[\s\S]+?)[\\\'|\"]/i';
                    preg_match($imageDivRegx, $responseHtml, $image);
                    if($image && $image['image']){
                        $productPageContent['image'] = $image['image'];
                    }

                    $productNameDivRegx = '/<div[\s]*?class=[\\\'|\"]col-md-12 col-sm-12[\\\'|\"]>[\s]*?<p[\s]*?class=[\\\'|\"]padding-tb-15[\\\'|\"]>[\s]*?<b>(?<name>[\s\S]+?)<\/b>(?<nameContent>[\s\S]+?)<\/p>/i';
                    preg_match_all($productNameDivRegx, $responseHtml, $productName);
                    if($productName){
                        foreach($productName['nameContent'] as $key => $name){
                            $strName = $this->stripHtml($name);
                            if(substr($strName, 0, 1) == ':'){
                                $strName = str_replace(':', '', $strName);
                                switch($key){
                                    case 0:
                                        $productPageContent['productName'] = $strName;break;
                                    case 1:
                                        $productPageContent['IUPAC'] = $strName;break;
                                }
                            }
                        }
                    }

                    $chemicalInfoDivRegx = '/<div[\s]*?class=[\\\'|\"]col-md-6 col-sm-12[\\\'|\"]>(?<chemicalInfoDiv>[\s\S]+?)<\/div>/i';
                    preg_match_all($chemicalInfoDivRegx, $responseHtml, $chemicalInfoDiv);
                    if($chemicalInfoDiv){
                        foreach($chemicalInfoDiv['chemicalInfoDiv'] as $div){
                            $div = $this->stripHtml($div);
                            $infoList = explode("\r\n",$div);
                            foreach($infoList as $info){
                                $info = trim($info);
                                if(empty($info)) continue;
                                try{
                                    list($name, $content) = explode('：', $info);
                                }catch(\Throwable $exception){
                                    list($name, $content) = explode(':', $info);
                                }
                                $productPageContent['casInfo'][$name] = $content;
                            }
                        }
                    }
                    //$productPageContent['casInfo'] = array_unique($productPageContent['casInfo'],SORT_REGULAR);

                    $productListDivRegx = '/<div[\s]*?class=[\\\'|\"]col-md-12 col-lg-12 col-sm-12 product-details-size[\\\'|\"]>[\s\S]*?<tbody>(?<productList>[\s\S]*?)<\/tbody>/i';
                    if(preg_match_all($productListDivRegx, $responseHtml, $productList)){
                        if(!empty($productList['productList'])){
                            foreach($productList['productList'] as $product){
                                $product = $this->stripTabTag($product);
                                preg_match_all('/<td[\s\S]*?>(?<content>[\s\S]*?)<\/td>/i', $product, $productContentTd);
                                $productPageContent['productList'] = [];
                                if($productContentTd['content']){
                                    $productContent = $productContentTd['content'];
                                    $productContent = array_chunk($productContent, 5);
                                    foreach($productContent as $content){
                                        if(empty($content[0])) continue;
                                        $productPageContent['productList'][] = ['productUnit' => $content[0], 'stock' => $content[1]];
                                    }
                                }
                            }
                        }
                    }
                    $this->apiService->saveProduct($productPageContent);
                }
            }
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), 'chemenu error', 'error');
        }
    }
}
