<?php

namespace App\Services\ThirdPlatform;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Rhawn\Services\RhawnProductService;
use App\Services\ThirdPlatform\Api\ChemSrcApiService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\Helpers;
use Illuminate\Support\Facades\DB;

class ChemSrcService
{
    use Helpers;

    public function __construct()
    {
        $this->apiService = (new ChemSrcApiService(app(ThirdApiPlatformRepository::class)));
        $this->rhawnChemicalsService = app(RhawnProductService::class);
    }


    public function batchUpdateCas($updateData)
    {
        if(empty($updateData['cas'])){
            return false;
        }
        $cas = $updateData['cas'];
        $molId = $updateData['mol_id'];
        try{
            $responseHtml = $this->apiService->getCasWebInfo($cas);
            if($responseHtml){
                $wuhuaDivRegx = '/id=[\\\'|\"]wuHuaDiv[\\\'|\"]>(?<wuhuaHtml>[\s\S]+?)<div\s{0,}class=[\\\'|\"]cata-div row[\\\'|\"]/i';
                preg_match($wuhuaDivRegx, $responseHtml,$wuhuaHtml);
                if(!empty($wuhuaHtml)) {
                    $wuhuaList = [];
                    $wuhuaDetailRegx =  '/<th[\s\S]*?>(?<title>[\s\S]*?)<\/th>[\s\S]*?<td[\s\S]*?>(?<content>[\s\S]*?)<\/td>/i';
                    preg_match_all($wuhuaDetailRegx, $wuhuaHtml['wuhuaHtml'],$wuhuaDetailList);
                    if(!empty($wuhuaDetailList)){
                        foreach($wuhuaDetailList['title'] as $key => $title){
                            $wuhuaList[] = ["key" => $title, "value" => str_replace( " ", "", str_replace( "\r\n", "", $wuhuaDetailList['content'][$key]))];
                            //$wuhuaList[$title] = str_replace( " ", "", str_replace( "\r\n", "", $wuhuaDetailList['content'][$key]));
                        }
                    }
                    if(!empty($wuhuaList)){
                        $baikeMysql = DB::connection('baike_mysql');
                        $baikeMysql->table('tp_mol_wuhua')
                            ->updateOrInsert([
                                'cas' => $cas
                            ], [
                                'mol_id' => $molId,
                                'content' => json_encode($wuhuaList, JSON_UNESCAPED_UNICODE)
                            ]);
                        SimpleLogs::writeLog(['cas' => $cas, 'content' => json_encode($wuhuaList, JSON_UNESCAPED_UNICODE)], 'mysql_data_insert');
                    }
                }
                sleep(rand(2,4));
                return true;
            }
        }catch(\Throwable $exception){
            /*if(stripos($exception->getMessage(), '403')){

            }*/
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateCas', 'error');
        }
    }

    public function notWuhuaCasList()
    {
        try{
            $status = $this->apiService->checkPlatformStatus();
            if($status){
                $limit = 100;
                $page = 0;

                ini_set('memory_limit','3072M');
                while(true) {
                    $baikeMysql = DB::connection('baike_mysql');
                    $casList = $baikeMysql->table('tp_mol_data')
                        ->offset($page * $limit)
                        ->limit($limit)
                        ->get()->toArray();
                    if (!$casList) {
                        break;
                    }

                    $waitSearchCasList = array_unique(array_column($casList, 'cas'));
                    $waitSearchCasListSort = array_column($casList, null, 'cas');

                    $wuhuaList = $baikeMysql->table('tp_mol_wuhua')
                        ->whereIn('cas', $waitSearchCasList)
                        ->get()->toArray();
                    $wuhuaCasList = array_unique(array_column($wuhuaList, 'cas'));

                    $waitGetCasWuhuaList = array_diff($waitSearchCasList, $wuhuaCasList);
                    if(!empty($waitGetCasWuhuaList)) {
                        foreach ($waitGetCasWuhuaList as $cas) {
                            if (!empty($waitSearchCasListSort[$cas])) {
                                $this->apiService->pushQueue([
                                    'params' => ['cas' => $cas, 'mol_id' => $waitSearchCasListSort[$cas]->id],
                                    'consumer' => __CLASS__,
                                    'method' => 'batchUpdateCas'
                                ], 'chemsrc_wuhua_crawl');
                            }
                        }
                    }
                    $page ++;
                }
            }
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':notWuhuaCasList', 'error');
        }
    }
}
