<?php


namespace App\Services\ThirdPlatform;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Rhawn\Services\RhawnProductService;
use App\Services\Kafka\KafkaService;
use App\Services\ThirdPlatform\Api\IntegleApiService;
use App\Support\Facades\SimpleKafka;
use App\Support\Facades\SimpleLogs;

class InteglePlatformService
{

    public function __construct()
    {
        $this->apiService = (new IntegleApiService(app(ThirdApiPlatformRepository::class)));
    }

    /**
     * 批量更新产品到integle平台
     */
    public function batchUpdateProducts($params = null)
    {
        $status = $this->apiService->checkPlatformStatus();
        if($status){
            $rhawnChemicalsService = app(RhawnProductService::class);
            $limit = $this->apiService->getPlatformInfo('platform_params')['batchNums'];
            $page = 0;

            while(true) {
                $rawList = $rhawnChemicalsService->getRawsList($page * $limit, $limit);
                if (!$rawList) {
                    break;
                }
                $rawIdList = [];
                foreach ($rawList as $raw) {
                    $rawIdList[] = $raw['r_id'];
                }
                $packages = $rhawnChemicalsService->getChemicalPackage($rawIdList);
                $newPackages = [];
                if($packages){
                    foreach($packages as $pack){
                        $newPackages[$pack['r_code']] = $pack;
                    }
                }
                if(!empty($rawList)){
                    $this->apiService->pushQueue([
                        'params' => ['rawList' => $rawList,'packageList' => $newPackages],
                        'consumer' => __CLASS__,
                        'method' => 'batchUpdateApi'
                    ],'integle');
                    $page ++;
                }
            }
        }
    }

    public function batchUpdateApi($updateData)
    {
        try{
            $result = $this->apiService->pushBatchUpdateProduct($updateData);
            unset($result['status']);

            $logsContent = [
                'apiParams' => $updateData,
                'apiReturn' => $result
            ];
            SimpleLogs::writeLog($logsContent,__CLASS__.':batchUpdateApi');
        }catch(\Throwable $exception){
            $this->apiService->pushQueue([
                'consumer' => __CLASS__,
                'method' => 'batchUpdateApi',
                'params' => $updateData
            ],'integle');
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':batchUpdateApi', 'error');
        }
    }
}
