<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services\ThirdPlatform;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Rhawn\Services\BhuaProductService;
use App\Services\ThirdPlatform\Api\LeyanApiService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\Helpers;
use Illuminate\Support\Facades\DB;

class LeyanService
{
    public function __construct()
    {
        $this->apiService = (new LeyanApiService(app(ThirdApiPlatformRepository::class)));
    }

    public function updateProductsByEveryDay()
    {
        $currentPage = 1;
        while (true) {
            $result = $this->apiService->pushBatchUpdateProductByPage($currentPage, 1000, $this->getToken());
            if (!$result || !$result['productInfo']) {
                return null;
            }
            try {
                $this->storeLeyanProducts($result['productInfo']);
                if ($result['allPage'] == $currentPage) {
                    break;
                }
                $currentPage++;
            } catch (\Exception $exception) {
                SimpleLogs::writeLog($exception->getMessage(), __CLASS__ . ':updateProductsByEveryDay', 'error');
                $currentPage++;
            }
        }
    }

    public function batchUpdateProducts($params = null)
    {
        ini_set('memory_limit', '3072M');
        $casList = app(BhuaProductService::class)->getProductCasByBidToGroupByCas($params);
        if ($casList) {
            foreach ($casList as $cas) {
                if (empty($cas['c_cas'])) continue;
                $this->apiService->pushQueue([
                    'params' => ['cas' => $cas['c_cas']],
                    'consumer' => __CLASS__,
                    'method' => 'updateProductByCas'
                ], 'leyan');
            }
        }
    }

    public function updateProductByCas($params)
    {
        if (!$params['cas']) return null;

        $result = $this->apiService->pushUpdateProductByCas($params['cas'], $this->getToken());
        if (!$result) {
            return null;
        }
        try {
            $result = $this->storeLeyanProducts($result);
        } catch (\Exception $exception) {
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__ . ':updateProductByCas', 'error');
        }
    }

    /**
     * @ 获取api token
     * @return false|mixed
     */
    public function getToken()
    {
        try {
            $token = $this->apiService->getPlatformDataEntries('leyan_api_token', 'data_values');
            if (empty($token)) {
                $apiToken = $this->apiService->getToken();
                if ($apiToken) {
                    $storeToken = [];
                    $storeToken['token'] = $apiToken;
                    $storeToken['currentTime'] = time();
                    $this->apiService->storePlatformDataEntries('leyan_api_token', $storeToken);
                    return $storeToken['token'];
                }
            } else {
                if ((time() - $token['currentTime']) >= 86400) {
                    $apiToken = $this->apiService->getToken();
                    if ($apiToken) {
                        $storeToken = [];
                        $storeToken['token'] = $apiToken;
                        $storeToken['currentTime'] = time();
                        $this->apiService->storePlatformDataEntries('leyan_api_token', $storeToken);
                        return $storeToken['token'];
                    }
                } else {
                    return $token['token'];
                }
            }
        } catch (\Exception $exception) {
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__ . ':token error', 'error');
            return false;
        }
    }


    /**
     * 保存bjs平台的产品数据
     */
    public function storeLeyanProducts($products)
    {
        if (!is_array($products) || empty($products)) {
            return false;
        }

        $bId = 108;

        $dbConnect = DB::connection('bh_mysql');

        $errorProduct = [];

        foreach ($products as $product) {

            try {

                $id = 0;

                $dbConnect->beginTransaction();

                $chemicalsRow = $dbConnect->table('chemicals')
                    ->where('c_cas', $product['casNo'])
                    ->get()->toArray();
                if (empty($chemicalsRow)) {
                    array_push($errorProduct, $product['casNo']);
                    continue;
                }
                $chemicalsRow = current($chemicalsRow);

                $dbConnect->table('products')
                    ->where('c_id', $chemicalsRow->c_id)
                    ->where('b_id', $bId)
                    ->update(['modifytime' => time()]);

                foreach ($product['productStockPrices'] as $package) {
                    //$p_tod = '无货，请咨询';
                    $p_tod = '现货';
                    $status = 1;
                    /*if (!empty($package['productStockInfos'])) {
                        foreach ($package['productStockInfos'] as $stock) {
                            if (isset($stock['city'])) {
                                if ($stock['city'] == '上海' && isset($stock['number']) && $stock['number'] > 0) {
                                    $status = 1;
                                    break;
                                }
                            }
                        }
                    }*/

                    $p_code = $product['catalogNo'] . '-' . $package['size'];

                    $productRow = $dbConnect->table('products')
                        ->where('b_id', $bId)
                        ->where('c_id', $chemicalsRow->c_id)
                        ->where('p_code', 'like', $p_code)
                        ->get()->toArray();

                    $pro_data = [];
                    if (empty($productRow)) {

                        if ($status == 0) {
                            SimpleLogs::writeLog('Leyan  cas : ' . $chemicalsRow->c_cas . '为下架状态不做更新', __CLASS__ . ':storeLeyanProducts');
                            continue;
                        }

                        $pro_data['b_id'] = $bId;
                        $pro_data['c_id'] = $chemicalsRow->c_id;
                        $pro_data['p_code'] = $p_code;
                        if (!$product['productNameCn']) {
                            $pro_data['p_cn_name'] = '';
                        } else {
                            $pro_data['p_cn_name'] = $product['productNameCn'];
                        }
                        if (!$product['productNameEn']) {
                            $pro_data['p_en_name'] = '';
                        } else {
                            $pro_data['p_en_name'] = $product['productNameEn'];
                        }
                        if (isset($product['purity']) && !empty($product['purity'])) {
                            $pro_data['p_level'] = $product['purity'];
                        }
                        $pro_data['p_pack'] = $package['size'];
                        if (isset($package['price']) && !empty($package['price'])) {
                            $pro_data['p_price'] = $package['price'];
                        }else{
                            SimpleLogs::writeLog('Leyan  cas : ' . $chemicalsRow->c_cas . ',p_code :'.$p_code.' 价格为空,不做更新', __CLASS__ . ':storeLeyanProducts');
                            continue;
                        }
                        $pro_data['p_tod'] = $p_tod;
                        $pro_data['p_weight'] = '0.01';
                        $pro_data['p_status'] = 1;//$status;
                        $pro_data['p_show'] = 1;
                        $pro_data['prom_from'] = 0;
                        $pro_data['prom_to'] = 0;
                        $pro_data['modifytime'] = time();

                        $id = $dbConnect->table('products')->insertGetId($pro_data);

                    } else {

                        $productRow = current($productRow);

                        $pro_data['p_status'] = 1;//$status;
                        $pro_data['p_show'] = 1;
                        if (isset($package['price']) && !empty($package['price'])) {
                            $pro_data['p_price'] = $package['price'];
                        }else{
                            $pro_data['p_price'] = $productRow->p_price;
                        }
                        $pro_data['p_tod'] = $p_tod;
                        $pro_data['modifytime'] = time();
                        $dbConnect->table('products')
                            ->where('p_id', $productRow->p_id)
                            ->update($pro_data);

                        $id = $productRow->p_id;
                    }
                }

                if ($id > 0) {
                    SimpleLogs::writeLog('Leyan p_id : ' . $id . ' cas : ' . $chemicalsRow->c_cas . ' 更新成功，更新时间:' . date('Y-m-d H:i:s', time()), __CLASS__ . ':storeLeyanProducts');
                }

                $dbConnect->commit();
            } catch (\Exception $exception) {
                SimpleLogs::writeLog($exception->getMessage(), __CLASS__ . ':storeLeyanProducts', 'error');
                $dbConnect->rollback();
                throw $exception;
            }
        }
        return true;
    }
}
