<?php

namespace App\Services\ThirdPlatform;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class PlatformBaseService extends PlatformAbstractService
{
    use \App\Support\Traits\Helpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
    }

    /**
     * 注册第三方api平台信息
     * @param $registerParams
     */
    public function registerPlatform($registerParams)
    {
        try{
            DB::beginTransaction();

            $thirdApiPlatformRepo = app(\App\Repositories\Eloquent\ThirdApiPlatformRepositoryEloquent::class);
            $result = $thirdApiPlatformRepo->findWhere(['platform_name' => $registerParams['platform_name']])->toArray();
            if(is_array($registerParams['platform_params'])){
                $registerParams['platform_params'] = json_encode($registerParams['platform_params'],JSON_UNESCAPED_UNICODE);
            }
            if(!isset($registerParams['platform_icon']) || empty($registerParams['platform_icon'])){
                $platform['platform_icon'] = '';
            }
            if(!$result){
                $platform['platform_status'] = '1';
                $platform['platform_token'] = $this->randomFromDevice(16);
                $platform['created_at'] = date('Y-m-d H:i:s',time());
                $platform['updated_at'] = date('Y-m-d H:i:s',time());
            }

            $thirdApiPlatformRepo->updateOrCreate(['platform_name' => $registerParams['platform_name']],array_merge($registerParams,$platform));

            SimpleLogs::writeLog($platform, __CLASS__.':'.__FUNCTION__);

            DB::commit();

            return true;

        }catch(\Exception $e){
            SimpleLogs::writeLog($e->getMessage(), __CLASS__.':'.__FUNCTION__, 'error');
            DB::rollback();

            return false;
        }
    }

}
