<?php

namespace App\Services\ThirdPlatform;

use App\ImportExport\BaseImport;
use App\ImportExport\WuxiLab\Import\Models\WuxiLabProductsImportModel;
use App\ImportExport\WuxiLab\Import\WuxiLabUpdateProductsImport;
use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Rhawn\Services\RhawnProductService;
use App\Services\ThirdPlatform\Api\WuxiApiNewService;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class WuxiLabNewService
{
    public function __construct()
    {
        $this->apiService = (new WuxiApiNewService(app(ThirdApiPlatformRepository::class)));
        $this->rhawnChemicalsService = app(RhawnProductService::class);
    }

    public function initToken()
    {
        return $this->getToken();
    }

    /**
     * 批量更新商品
     */
    public function batchUpdateProducts($type = 'all')
    {
        $status = $this->apiService->checkPlatformStatus();
        if($status){
            $limit = $this->apiService->getPlatformInfo('platform_params')['batchNums'];
            $page = 0;

            ini_set('memory_limit','3072M');
            while(true) {
                $rawList = $this->rhawnChemicalsService->getRawsList($page * $limit, $limit);
                if (!$rawList) {
                    break;
                }
                $rawIdList = [];
                foreach ($rawList as $raw) {
                    $rawIdList[] = $raw['r_id'];
                }
                $packages = $this->rhawnChemicalsService->getChemicalPackage($rawIdList);
                $newPackages = [];
                if($packages){
                    foreach($packages as $pack){
                        //符合110状态才上架
                        if($pack['p_status'] == 1 && $pack['p_show'] == 1 && $pack['p_if_big'] == 0){
                            $newPackages[$pack['p_code']] = $pack;
                        }
                        //$newPackages[$pack['r_code']] = $pack;
                    }
                }

                $queueParams = [];
                switch($type){
                    case 'all': $queueParams = [
                        'params' => ['rawList' => $rawList,'packageList' => $newPackages],
                        'consumer' => __CLASS__,
                        'method' => 'queueBatchUpdateProductToApi'
                    ];break;
                    case 'package': $queueParams = [
                        'params' => ['packageList' => $newPackages],
                        'consumer' => __CLASS__,
                        'method' => 'queueBatchUpdatePackagesToApi'
                    ];break;
                    case 'init_package': $queueParams = [
                        'params' => ['packageList' => $newPackages],
                        'consumer' => __CLASS__,
                        'method' => 'initQueueBatchUpdatePackagesToApi'
                    ];break;
                    default:
                        $queueParams = [
                            'params' => ['rawList' => $rawList,'packageList' => $newPackages],
                            'consumer' => __CLASS__,
                            'method' => 'queueBatchUpdateProductToApi'
                        ];
                }

                if(!empty($rawList)){
                    $this->apiService->pushQueue($queueParams,'wuxilab_api');
                }
                $page ++;
            }
        }
    }

    /**
     * 批量更新产品价格
     * @return void
     * @throws \Exception
     */
    public function batchUpdateProductPrice()
    {
        try{
            $status = $this->apiService->checkPlatformStatus();
            if($status){
                $dbConnect = DB::connection('rhawn_mysql');

                $startime = date('Y-m-d', strtotime('-1 day')).' 00:00:00';
                $endtime = date('Y-m-d', time()).' 00:00:00';

                $product = $dbConnect->table('prochange')
                    ->whereRaw("updatetime >=".strtotime($startime)." and updatetime <=".strtotime($endtime))
                    ->where('type', '2')
                    ->get()->toArray();
                if(count($product) > 0){
                    $productIds = array_unique(array_column($product, 'p_id'));
                    $productList = $this->rhawnChemicalsService->getProductsListThroughPids($productIds);

                    $updateProductPrice = [];
                    foreach($productList as $product){
                        $tempData = [];
                        $tempData['product_division'] = 'CHEMICAL';
                        list($pCode, $packCode) = explode('-', $product->p_code);
                        $tempData['productId'] = $pCode;
                        $tempData['packageUnitDescription'] = $product->p_code;
                        $tempData['minPackageUnitDescription'] = '';
                        $tempData['price'] = bcmul(ceil($product->p_price), 0.5, 2);
                        $tempData['discount'] = '0.5';
                        $tempData['inCostCurrency'] = 'RMB';
                        $tempData['status'] = $product->p_status == '1' ? 'ACTIVE' : 'INACTIVE';;
                        $updateProductPrice[] = $tempData;
                    }
                    $result = $this->apiService->pushBatchUpdatePrice($updateProductPrice, $this->getToken());
                    $this->processApiResponse($result);
                }
            }
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateToWuxiLab', 'error');
        }
    }

    public function batchUpdateProductStock()
    {
        try{
            $status = $this->apiService->checkPlatformStatus();
            if($status){
                $dbConnect = DB::connection('rhawn_mysql');

                $startime = date('Y-m-d', strtotime('-1 day')).' 00:00:00';
                $endtime = date('Y-m-d', time()).' 00:00:00';

                $product = $dbConnect->table('prochange')
                    ->whereRaw("updatetime >=".strtotime($startime)." and updatetime <=".strtotime($endtime))
                    ->where('type', '1')
                    ->get()->toArray();
                if(count($product) > 0){
                    $productIds = array_unique(array_column($product, 'p_id'));
                    $productList = $this->rhawnChemicalsService->getProductsListThroughPids($productIds);

                    $updateProductStock = [];
                    foreach($productList as $product){
                        $tempData = [];
                        $tempData['product_division'] = 'CHEMICAL';
                        list($pCode, $packCode) = explode('-', $product->p_code);
                        $tempData['productId'] = $pCode;
                        $tempData['packageUnitDescription'] = $product->p_code;
                        $tempData['packageStockQuantity'] = $product->p_stock;
                        $tempData['packageStockLocation'] = 'CN-SH';
                        $tempData['stockStatus'] = $product->p_status == '1' ? 'ACTIVE' : 'INACTIVE';;
                        $updateProductStock[] = $tempData;
                    }
                    $result = $this->apiService->pushBatchUpdateStock($updateProductStock, $this->getToken());
                    $this->processApiResponse($result);
                }
            }
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateToWuxiLab', 'error');
        }
    }

    public function initQueueBatchUpdatePackagesToApi($updateData)
    {
        if(empty($updateData['packageList'])){
            return false;
        }
        try{
            $result = $this->apiService->pushBatchUpdatePackages($updateData['packageList'], $this->getToken());
            $this->processApiResponse($result);


            $result = $this->apiService->pushBatchUpdateStock($updateData['packageList'], $this->getToken());
            $this->processApiResponse($result);

            $result = $this->apiService->pushBatchUpdatePrice($updateData['packageList'], $this->getToken());
            $this->processApiResponse($result);

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateToWuxiLab', 'error');
        }
    }

    public function queueBatchUpdatePackagesToApi($updateData)
    {
        if(empty($updateData['packageList'])){
            return false;
        }
        try{
            $result = $this->apiService->pushBatchUpdatePackages($updateData['packageList'], $this->getToken());
            $this->processApiResponse($result);

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateToWuxiLab', 'error');
        }
    }

    /**
     * @批量更新商品
     * @param $updateData
     */
    public function queueBatchUpdateProductToApi($updateData)
    {
        if(empty($updateData['rawList']) || empty($updateData['packageList'])){
            return false;
        }
        try{
            $noSmiles = [];
            foreach($updateData['rawList'] as $data){
                if(empty($data['c_smiles']) && !empty($data['c_cas'])){
                    $noSmiles[] = $data['c_cas'];
                }
            }
            $updateData['rawList'] = array_column($updateData['rawList'], null, 'c_cas');
            $productsIdsList = array_column($updateData['rawList'], 'r_code');

            if(!empty($noSmiles)){
                $noSmiles = array_unique($noSmiles);
                $baikeMysql = DB::connection('baike_mysql');
                $casResult = $baikeMysql->table('tp_mol_data')->whereIn('cas', $noSmiles)->get()->toArray();
                if($casResult){
                    foreach($casResult as $cas){
                        $updateData['rawList'][$cas->cas]['c_smiles'] = $cas->smiles;
                    }
                }
            }

            $productsList = app(WuxiLabUpdateProductsService::class)->getUpdateProductsListByProductIds($productsIdsList);
            if(!empty($productsList)){
                $productsList = array_column($productsList, null, 'product_id');
                foreach($updateData['rawList'] as $key => &$data){
                    if(empty($productsList[$data['r_code']])){
                        unset($updateData['rawList'][$key]);
                        continue;
                    }
                    $data['product_extend'] = $productsList[$data['r_code']];
                }
            }

            $result = $this->apiService->pushBatchUpdateProduct($updateData, $this->getToken());
            $this->processApiResponse($result);

            sleep(rand(2,5));

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateToWuxiLab', 'error');
        }
    }

    protected function processApiResponse($apiResponse)
    {
        if($apiResponse){
            //处理更新不成功的商品
            if(!empty($apiResponse['errorProduct'])){
                $errorProductsList = $this->apiService->getPlatformDataEntries('wuxiLab_error_products','data_values');
                foreach($apiResponse['errorProduct'] as $errorProduct){
                    if(!is_null($errorProductsList) && !empty($errorProductsList['products'])){
                        if(!in_array($errorProduct['data'],$errorProductsList['products'])){
                            array_push($errorProductsList['products'],$errorProduct['data']);
                        }
                    }else{
                        $errorProductsList['products'][] = $errorProduct['data'];
                    }
                }
                $this->apiService->storePlatformDataEntries('wuxiLab_error_products',['products' => $errorProductsList['products']]);
            }
            if(!empty($apiResponse['errorPackage'])) {
                $errorPackagesList = $this->apiService->getPlatformDataEntries('wuxiLab_error_packages', 'data_values');
                //更新成功的商品
                foreach ($apiResponse['errorPackage'] as $errorPackage) {
                    if (!is_null($errorPackagesList) && !empty($errorPackagesList['products'])) {
                        if (!in_array($errorPackage['data'], $errorPackagesList['products'])) {
                            array_push($errorPackagesList['products'], $errorPackage['data']);
                        }
                    } else {
                        $errorPackagesList['products'][] = $errorPackage['data'];
                    }
                }
                $this->apiService->storePlatformDataEntries('wuxiLab_error_packages', ['products' => $errorPackagesList['products']]);


            }

            if(!empty($apiResponse['successPackage'])) {
                $updateSuccessPackagesList = $this->apiService->getPlatformDataEntries('wuxiLab_update_success_packages', 'data_values');
                //更新成功的商品
                foreach ($apiResponse['successPackage'] as $successProduct) {
                    if (!is_null($updateSuccessPackagesList) && !empty($updateSuccessPackagesList['products'])) {
                        if (!in_array($successProduct, $updateSuccessPackagesList['products'])) {
                            array_push($updateSuccessPackagesList['products'], $successProduct);
                        }
                    } else {
                        $updateSuccessPackagesList['products'][] = $successProduct;
                    }
                }
                $this->apiService->storePlatformDataEntries('wuxiLab_update_success_packages', ['products' => $updateSuccessPackagesList['products']]);
            }
        }
    }

    /**
     * @ 获取api token
     * @return false|mixed
     */
    public function getToken(){
        try{
            $token = $this->apiService->getPlatformDataEntries('wuxilab_token','data_values');
            if(empty($token) || (time() > $token['ttl'])){
                $token = $this->apiService->getToken();
                if($token && $token['code'] == 200){
                    $this->apiService->storePlatformDataEntries('wuxilab_token',['token' => $token['data'], 'ttl' => strtotime('+2 hour')]);
                    return $token['data'];
                }
            }
            return $token['token'];
        }catch(\Exception $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':token error', 'error');
            return false;
        }
    }

    public function initNeedUpdateProductsList($params)
    {
        $importService = app(WuxiLabUpdateProductsImport::class);
        $importRowsList = $importService->loadImportExcel($params);
        if($importRowsList){
            foreach($importRowsList as $key => $row){
                if($key == 0){
                    continue;
                }
                app(WuxiLabUpdateProductsService::class)->saveUpdateProducts($row, $importService->getTitleHeader());
            }
        }
    }
}
