<?php

namespace App\Services\ThirdPlatform;

use App\Mailer\MailService;
use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Rhawn\Services\RhawnProductService;
use App\Services\Kafka\KafkaService;
use App\Services\ThirdPlatform\Api\ZhenkhApiService;
use App\Support\Facades\SimpleKafka;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\Helpers;

class ZhenKhService
{
    use Helpers;

    public function __construct()
    {
        $this->apiService = (new ZhenkhApiService(app(ThirdApiPlatformRepository::class)));
        $this->rhawnService = app(RhawnProductService::class);
        $this->zhenKhOrderService = app(\App\Rhawn\Services\ZhenkhService::class);
        $this->mailService = app(MailService::class);
    }

    public function createJwtToken()
    {
        $token = $this->apiService->getPlatformInfo('platform_token');
        $jwtToken = $this->generateToken($token);
        var_dump($jwtToken);
        exit;
    }

    /**
     * @上报商品的价格及库存
     * @param $params
     * @return false|void
     */
    public function consumerUpdateReportPriceAndStock($params)
    {
        if(!isset($params['goods']) || empty($params['goods'])){
            return false;
        }
        $goods = $params['goods'];

        try{
            $product = $this->rhawnService->getProductsThroughPid($goods->zg_r_pid);
            if($product){
                while(true){
                    $result = $this->apiService->updateReportProductPrice([
                        'price' => round(bcmul($product->p_price,0.5,2)),
                        'price_unit' => $goods->zg_unit,
                        'sku' => $goods->zg_sku
                    ],$this->getToken());
                    if(!isset($result['reset']) || $result['reset'] != true){
                       break;
                    }
                    if(!$result){
                        break;
                    }
                }

                while(true){
                    $result = $this->apiService->updateReportProductStock([
                        'warehouse' => '上海货仓',
                        'address' => ['0' => $goods->zg_addr],
                        'qty' => $product->p_stock,
                        'sku' => $goods->zg_sku
                    ],$this->getToken());
                    if(!isset($result['reset']) || $result['reset'] != true){
                        break;
                    }
                    if(!$result){
                        break;
                    }
                }
            }
            $result = $this->zhenKhOrderService->updateSyncRunTime($goods->zg_id);
            if($result){
                SimpleLogs::writeLog($goods->zg_sku.'的价格和库存上报完成。完成时间：'.date('Y-m-d H:i:s',time()),__CLASS__.':consumerUpdateReportPriceAndStock');
            }

        }catch (\Throwable $exception) {
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $goods,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
        }
    }

    /**
     * @更新供应商商品
     * @throws \Throwable
     */
    public function getUpdateSupplierGoods()
    {
        try{
            $page = 1;
            $sendContent = [];
            while(true){
                $pageSize = $this->apiService->getPlatformInfo()['platform_params']['page_size'];
                while(true){
                    $goodsList = $this->apiService->getUpdateGoods($page,$pageSize,$this->getToken());
                    if(!isset($goodsList['reset']) || $goodsList['reset'] != true){
                        break;
                    }
                }
                if($goodsList && count($goodsList['list']) > 0){
                    foreach($goodsList['list'] as $goods){
                        $sendContent = [
                            'params' => ['goods' => $goods],
                            'consumer' => __CLASS__,
                            'method' => 'consumerBatchUpdateGoodsList'
                        ];
                        $this->apiService->pushQueue($sendContent,'zkh');
                    }
                }

                $page ++;
                if($page > $goodsList['total_page']){
                    break;
                }
            }

        }catch(\Throwable $exception){
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $sendContent,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
            throw $exception;
        }
    }

    /**
     * @获取订单列表
     * @throws \Throwable
     */
    public function getUpdateSupplierOrders()
    {
        try{
            $page = 1;
            $sendContent = [];
            $postParams = [
                'start_time' => date('Y-m-d H:i:s',(time() - 86400)),
                'end_time' => date('Y-m-d H:i:s',strtotime(date('Y-m-d',time())." 23:59:59")),
                'page_size' => 0,
                'page_no' => 0,
            ];
            while(true){
                $pageSize = $this->apiService->getPlatformInfo()['platform_params']['page_size'];
                $postParams['page_size'] = $pageSize;
                $postParams['page_no'] = $page;
                while(true){
                    $orderList = $this->apiService->getUpdateOrders($postParams,$this->getToken());
                    if(!isset($orderList['reset']) || $orderList['reset'] != true){
                        break;
                    }
                }
                if($orderList && count($orderList['list']) > 0){
                    foreach($orderList['list']['data'] as $order){
                        if($order['orderStatus'] == 0){
                            $this->apiService->pushQueue([
                                'params' => ['orders' => $order],
                                'consumer' => __CLASS__,
                                'method' => 'consumerBatchUpdateOrdersList'
                            ],'zkh');
                        }
                    }
                }

                $page ++;
                if($page > $orderList['total_page']){
                    break;
                }
            }

        }catch(\Throwable $exception){
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $sendContent,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
            throw $exception;
        }
    }


    /**
     * 消费批量更新商品
     * @param $params
     */
    public function consumerBatchUpdateGoodsList($params)
    {
        if(!isset($params['goods']) || empty($params['goods'])){
            return false;
        }
        $goods = $params['goods'];

        try {
            $result = $this->zhenKhOrderService->updateZhenkhGoods($goods);
            if(!is_null($result)){
                $this->consumerUpdateReportPriceAndStock(['goods' => $result]);
                //更新商品的价格和库存
                /*SimpleKafka::produerSend([
                    'params' => ['goods' => $result],
                    'consumer' => __CLASS__,
                    'method' => 'consumerUpdateReportPriceAndStock'
                ],'zkh');*/
            }

        } catch (\Throwable $exception) {
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $goods,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
        }
    }

    /**
     * @更新订单信息
     * @param $params
     * @return false|void
     */
    public function consumerBatchUpdateOrdersList($params)
    {
        if(!isset($params['orders']) || empty($params['orders'])){
            return false;
        }

        $order = $params['orders'];
        try {
            if($order['orderStatus'] == 0){
                while(true){
                    $orderDetail = $this->apiService->getUpdateOrdersDetail($order['purchaseOrderId'],$this->getToken());
                    if(!isset($orderDetail['reset']) || $orderDetail['reset'] != true){
                        break;
                    }
                    if(!$orderDetail){
                        break;
                    }
                }
                $result = $this->zhenKhOrderService->updateZhenkhOrders($order,$orderDetail);
            }

        } catch (\Throwable $exception) {
            $this->sendMail('订单同步邮件', '订单号:'.implode(',',$order['orderNo']).'未同步成功');
            /*$this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $order,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);*/
        }
    }

    /**
     * 手动执行订单更新
     * @throws \Throwable
     */
    public function hMUpdateOrdersList()
    {
        try{
            $page = 1;
            $sendContent = [];
            $postParams = [
                'start_time' => date('Y-m-d H:i:s',(time() - 86400)),
                'end_time' => date('Y-m-d H:i:s',strtotime(date('Y-m-d',time())." 23:59:59")),
                'page_size' => 0,
                'page_no' => 0,
            ];
            while(true){
                $pageSize = $this->apiService->getPlatformInfo()['platform_params']['page_size'];
                $postParams['page_size'] = $pageSize;
                $postParams['page_no'] = $page;
                while(true){
                    $orderList = $this->apiService->getUpdateOrders($postParams,$this->getToken());
                    if(!isset($orderList['reset']) || $orderList['reset'] != true){
                        break;
                    }
                }
                if($orderList && count($orderList['list']) > 0){
                    foreach($orderList['list']['data'] as $order){
                        if($order['orderStatus'] == 0){
                            while(true){
                                $orderDetail = $this->apiService->getUpdateOrdersDetail($order['purchaseOrderId'],$this->getToken());
                                if(!isset($orderDetail['reset']) || $orderDetail['reset'] != true){
                                    break;
                                }
                            }
                            if($orderDetail){
                                $result = $this->zhenKhOrderService->updateZhenkhOrders($order,$orderDetail);
                            }
                        }
                    }
                }

                $page ++;
                if($page > $orderList['total_page']){
                    break;
                }
            }

        }catch(\Throwable $exception){
            if($exception->getCode() == '502'){
                $this->sendMail('订单同步邮件', '订单号:'.$order['purchaseOrderId'].'未同步成功');
            }else{
                $this->apiService->throwableAfter([
                    'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                    'queue_params' => $sendContent,
                    'fail_content' => $exception->getMessage(),
                    'queue_reset' => time() + 86400 * 10
                ]);
            }
            throw $exception;
        }
    }

    /**
     * 批量更新zkh发货单
     */
    public function batchUpdateDeliveryOrder($params)
    {
        $path = config('filesystems.disks.local.root').'/upload/'.$params;
        $generator = Helpers::yieldLoadCsv($path);

        // yield 循环逐行读取csv内容
        while ($generator->valid()) {
            $content = $generator->current();
            if(empty($content)){
                break;
            }
            $this->apiService->pushQueue([
                'params' => current($content),
                'consumer' => __CLASS__,
                'method' => 'sendDeliveryOrder'

            ],'zkh_updateDeliveryOrder');
            $generator->next();
        }
    }

    /**
     * @订单发货
     * @param $deliveryCode
     */
    public function sendDeliveryOrder($deliveryCode)
    {
        try{
            $orderDelivery = $this->zhenKhOrderService->getSorderDeliveryDetail($deliveryCode);
            $noMatchGoods = [];
            if($orderDelivery){
                $expressArray = $this->apiService->getPlatformDataEntries('zkh_order_express','data_values');
                if(empty($expressArray)){
                    while(true){
                        $expressList = $this->apiService->expressLogistics($this->getToken());
                        if(!isset($expressList['reset']) || $expressList['reset'] != true){
                            break;
                        }
                    }
                    if($expressList){
                        //$sysDefault = ['JD', 'SF', 'ZT', 'YD', 'ZJS', 'DBL', 'YTO', 'YZPY', 'AN', 'STO', 'HHTT', 'WL', 'ziti'];
                        foreach($expressList as $express){
                            $tempExpress = [];
                            $tempExpress['id'] = $express['id'];
                            $tempExpress['expressCode'] = $express['expressCode'];
                            $tempExpress['expressName'] = $express['expressName'];
                            $tempExpress['status'] = $express['status'];
                            $tempExpress['logisticsLinkType'] = $express['logisticsLinkType'];
                            switch($express['expressCode']){
                                case 'debangkuaidi' : $expressArray['DBL'] = $tempExpress; break;
                                case 'yuantong' : $expressArray['YTO'] = $tempExpress; break;
                                case 'yunda' : $expressArray['YD'] = $tempExpress; break;
                                case 'guoeryue' : $expressArray['HHTT'] = $tempExpress; break;
                                case 'shunfeng' : $expressArray['SF'] = $tempExpress; break;
                                case 'jd' : $expressArray['JD'] = $tempExpress; break;
                                case 'danniao' : $expressArray['CNSD'] = $tempExpress; break;
                            }
                        }
                        $this->apiService->storePlatformDataEntries('zkh_order_express',$expressArray);
                    }
                }
                $orderNo = current($orderDelivery)->so_cus_po;
                while(true){
                    $deliveryOrderDetail = $this->apiService->sendDeliveryOrder($orderNo,$this->getToken());
                    if(!isset($deliveryOrderDetail['reset']) || $deliveryOrderDetail['reset'] != true){
                        break;
                    }
                }
                $deliveryDetail = [];
                if($deliveryOrderDetail){
                    foreach($deliveryOrderDetail['lines'] as $order){
                        $deliveryDetail[$order['zkhSku']] = $order;
                    }
                }

                $deliveryPost = [];
                $currentOrderDelivery = current($orderDelivery);
                $deliveryPost['deliveryTime'] = date('Y-m-d',time());
                $deliveryPost['deliveryWay'] = $currentOrderDelivery->dp_express_com == 'ziti' || $currentOrderDelivery->dp_express_com == 'WL' ? 3 : 1;
                if($deliveryPost['deliveryWay'] == 1){
                    $zkhDelivery = $expressArray[$currentOrderDelivery->dp_express_com];
                    $deliveryPost['logisticsId'] = $zkhDelivery['id'];
                    $deliveryPost['logisticsName'] = $zkhDelivery['expressName'];
                    $deliveryPost['logisticsCode'] = str_replace('-',',',$currentOrderDelivery->dp_express_no);//$zkhDelivery['expressCode'];
                }
                $itemList = [];
                foreach($orderDelivery as $delivery){
                    if($delivery->p_id == 0) continue; //跳过运费
                    $zkhGoods = $this->zhenKhOrderService->getZkhGoodsSku($delivery->p_id);
                    if(empty($zkhGoods)){
                        $noMatchGoods[] = $orderNo;
                        throw new \Exception('对应震坤行商品数据未找到');
                    }
                    foreach($zkhGoods as $goods){
                        $temp = [];
                        if(isset($itemList[$goods->zg_sku]) && !empty($itemList[$goods->zg_sku])){
                            $itemList[$goods->zg_sku]['number'] = intval($itemList[$goods->zg_sku]['number']) + intval($delivery->dpd_num);
                        }else{
                            $temp['purchaseOrderId'] = $currentOrderDelivery->so_cus_po;
                            $temp['itemLineNumber'] = $deliveryDetail[$goods->zg_sku]['itemLineNumber'];
                            $temp['planItemNumber'] = $deliveryDetail[$goods->zg_sku]['planItemNumber'];
                            $temp['zkhSku'] = $goods->zg_sku;
                            $temp['materialDescribe'] = $deliveryDetail[$goods->zg_sku]['materialDescribe'];
                            $temp['number'] = $delivery->dpd_num;
                            $temp['unit'] = $deliveryDetail[$goods->zg_sku]['unit'];
                            $itemList[$goods->zg_sku] = $temp;
                        }
                    }
                }
                $resetItemList = [];
                if(!empty($itemList)){
                    foreach($itemList as $item){
                        $resetItemList[] = $item;
                    }
                }
                $deliveryPost['itemList'] = $resetItemList;
                while(true){
                    //确认发货
                    $deliveryResult = $this->apiService->ackDeliveryOrder($deliveryPost,$this->getToken());
                    if(!isset($deliveryResult['reset']) || $deliveryResult['reset'] != true){
                        break;
                    }
                }
                if($deliveryResult){
                    $this->apiService->storePlatformDataEntries('zkh_delivery_code_'.$deliveryCode,['code' => $deliveryResult['data']]);
                }
            }
            if(!empty($noMatchGoods)){
                //发送邮件
                $this->sendMail('订单同步邮件', '订单号:'.implode(',',$noMatchGoods).'未同步成功');
            }

        }catch(\Throwable $exception){
            if(!empty($noMatchGoods)){
                //发送邮件
                $this->sendMail('订单同步邮件', '订单号:'.implode(',',$noMatchGoods).'未同步成功 ,错误原因 ： '.$exception->getMessage());
            }
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':sendDeliveryOrder', 'error');
        }
    }

    /**
     * @获取订单送货单pdf
     * @param $deliveryCode
     * @return mixed|void
     */
    public function getDeliveryOrderPdf($deliveryCode)
    {
        $pdf = $this->apiService->getPlatformDataEntries('zkh_order_delivery_pdf_'.$deliveryCode,'data_values');
        if(!$pdf || empty($pdf)){
            $zkhDeliveryCode = $this->apiService->getPlatformDataEntries('zkh_delivery_code_'.$deliveryCode,'data_values');
            if(!$zkhDeliveryCode){
                SimpleLogs::writeLog($deliveryCode.'送货单无法匹配', __CLASS__.':getDeliveryOrderPdf', 'error');
                return false;
            }
            while(true){
                $deliveryInfo = $this->apiService->getDeliveryOrderDetailForPDF($zkhDeliveryCode['code']['deliveryCode'],$this->getToken());
                if(!isset($result['reset']) || $result['reset'] != true){
                    break;
                }
            }
            if($deliveryInfo){
                $this->apiService->storePlatformDataEntries('zkh_order_delivery_pdf_'.$deliveryCode,['pdf' => $deliveryInfo]);
                return $deliveryInfo;
            }
        }
        if(!is_null($pdf)){
            return $pdf['pdf'];
        }
        return false;
    }

    /**
     * @订单确认
     * @param $orderNo
     * @return bool|mixed|void
     */
    public function orderConfirm($orderNo)
    {
        try{
            $orderDetail = $this->zhenKhOrderService->getSorderDetail($orderNo);
            if($orderDetail){
                while(true){
                    $result = $this->apiService->orderConfirm(current($orderDetail)['so_cus_po'], $this->getToken());
                    if(!isset($result['reset']) || $result['reset'] != true){
                        break;
                    }
                    if(!$result){
                        //发送邮件
                        $this->sendMail('订单确认邮件', '订单号:'.$orderNo.'未确认成功');
                    }
                }
            }
            return false;
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':orderConfirm', 'error');
            return false;
        }
    }

    /**
     * @ 获取api token
     * @return false|mixed
     */
    public function getToken(){
        try{
            $token = $this->apiService->getPlatformDataEntries('zkh_api_token','data_values');
            if(empty($token)){
                $apiToken = $this->apiService->getToken();
                if($apiToken){
                    $apiToken['currentTime'] = time();
                    $this->apiService->storePlatformDataEntries('zkh_api_token',$apiToken);
                    return $apiToken['token'];
                }
            }else{
                //超过有效期，重新请求获取
                if(time() - intval($token['currentTime']) > $token['tokenTtl']){
                    $apiToken = $this->apiService->getToken();
                    if($apiToken){
                        $apiToken['currentTime'] = time();
                        $this->apiService->storePlatformDataEntries('zkh_api_token',$apiToken);
                        return $apiToken['token'];
                    }
                }
                return $token['token'];
            }
        }catch(\Exception $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':token error', 'error');
            return false;
        }
    }

    public function sendMail($title, $message)
    {
        try{
            $mailer = $this->apiService->getPlatformDataEntries('zkh_send_mailer','data_values');
            $this->mailService->sendMail($title, $mailer,'震坤行【'.$message.'】,请人工操作!');
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':sendMail', 'error');
            return false;
        }
    }

}
