<?php

namespace App\TestTools\Rhawn;

use App\Rhawn\Repositories\Eloquent\RhawnSoitemsRepositoryEloquent;
use App\Rhawn\Repositories\Eloquent\RhawnSordersRepositoryEloquent;
use App\Rhawn\Services\RhawnCustomerService;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class RhawnToolsService
{
    public function __construct(\App\Rhawn\Services\RhawnOrdersService $rhawnOrdersService)
    {
        $this->rhawnOrdersService = $rhawnOrdersService;
        $this->rhawnCustomerService = app(RhawnCustomerService::class);
    }

    /**
     * 修改用户订单价格
     */
    public function updateCustomerOrdersPricesByDiscount($cusCode, $discount)
    {
        $customer = $this->rhawnCustomerService->checkCustomerExist($cusCode);
        if($customer){
            $rhawnOrderRepository = app(RhawnSordersRepositoryEloquent::class);
            $orderList = $rhawnOrderRepository->getOrdersListThroughtUserId($customer['cus_id'],1,1000);
            if($orderList) {
                $armedOrders = [];
                foreach ($orderList as $order) {
                    if ($order->so_review_status == 0) {
                        array_push($armedOrders, $order);
                    }
                }
                $connection = DB::connection($rhawnOrderRepository->getConnectionName());
                $connection->beginTransaction();
                try {

                    $updateCount = 0;
                    //处理状态0的订单
                    if (count($armedOrders) > 0) {
                        foreach ($armedOrders as $order) {
                            $itemRepository = app(RhawnSoitemsRepositoryEloquent::class);
                            $items = $itemRepository->getSorderItemsDetailFromOrderId($order->so_id);
                            if ($items) {
                                foreach ($items as $detail) {
                                    $si_discount = 0;
                                    $si_amount = 0;
                                    if ($detail['p_id'] > 0) {
                                        $si_discount = round($detail['si_price'] * $discount);
                                        $si_amount = bcmul($detail['si_num'], $si_discount);
                                        $connection->table('soitems')
                                            ->where('si_id',$detail['si_id'])
                                            ->update([
                                                'si_discount' => $si_discount,
                                                'si_amount' => $si_amount
                                            ]);
                                    }
                                }
                                //更新订单总价
                                $total = $connection->table('soitems')
                                    ->where('so_id',$order->so_id)
                                    ->select(DB::raw('sum(si_amount) as amount'))
                                    ->get()->toArray();

                                $connection->table('sorders')
                                    ->where('so_id',$order->so_id)
                                    ->update([
                                        'so_total' => $total[0]->amount,
                                    ]);
                                $updateCount ++;
                                SimpleLogs::writeLog('订单号 : '.$order->so_id.'总价变更为:'.$total[0]->amount.'更新成功',__CLASS__.':updateCustomerOrdersPricesByDiscount');
                                SimpleLogs::writeLog('更新数量: '.$updateCount.'条数据',__CLASS__.':updateCustomerOrdersPricesByDiscount');
                            }
                        }
                        $connection->commit();
                    }
                }catch(\Throwable $exception){
                    $connection->rollBack();
                    SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':updateCustomerOrdersPricesByDiscount','error');
                }
            }
        }
    }
}
